# Removed some imports that were necessary only for implementation
import sys

import numpy as np
from pandas import Categorical as Categorical, DataFrame as DataFrame, Series as Series
from pandas._typing import FilePathOrBuffer as FilePathOrBuffer
from pandas.core.arrays.datetimes import DatetimeArray as DatetimeArray
from pandas.core.arrays.timedeltas import TimedeltaArray as TimedeltaArray
from pandas.core.base import PandasObject as PandasObject

from pandas.core.dtypes.generic import ABCIndexClass as ABCIndexClass, ABCMultiIndex as ABCMultiIndex, ABCSeries as ABCSeries, ABCSparseArray as ABCSparseArray
from pandas.core.dtypes.missing import isna as isna, notna as notna
from pandas.core.indexes.api import Index as Index, ensure_index as ensure_index
from pandas.core.indexes.datetimes import DatetimeIndex as DatetimeIndex
from pandas.core.indexes.timedeltas import TimedeltaIndex as TimedeltaIndex
from pandas.errors import AbstractMethodError as AbstractMethodError
from pandas.io.common import stringify_path as stringify_path
from pandas.io.formats.printing import adjoin as adjoin, justify as justify, pprint_thing as pprint_thing
from typing import Any, Callable, Dict, IO, Iterable, List, Mapping, Optional, Sequence, Tuple, Union

if sys.version_info >= (3, 8):
    from typing import Literal
else:
    from typing_extensions import Literal

formatters_type = Union[List[Callable[..., Any]], Tuple[Callable[..., Any], ...], Mapping[Union[str, int], Callable[..., Any]]]
float_format_type: Any
FloatFormatType = Union[str, Callable[..., Any]]
common_docstring: str

VALID_JUSTIFY_PARAMETERS = Literal["left", "right", "center", "justify", "justify-all",
                                   "start", "end", "inherit", "match-parent", "initial", "unset"]

return_docstring: str

class CategoricalFormatter:
    categorical: Any = ...
    buf: Any = ...
    na_rep: Any = ...
    length: Any = ...
    footer: Any = ...
    def __init__(self, categorical: Categorical, buf: Optional[IO[str]]=..., length: bool=..., na_rep: str=..., footer: bool=...) -> None: ...
    def to_string(self) -> str: ...

class SeriesFormatter:
    series: Any = ...
    buf: Any = ...
    name: Any = ...
    na_rep: Any = ...
    header: Any = ...
    length: Any = ...
    index: Any = ...
    max_rows: Any = ...
    min_rows: Any = ...
    float_format: Any = ...
    dtype: Any = ...
    adj: Any = ...
    def __init__(self, series: Series, buf: Optional[IO[str]]=..., length: Union[bool, str]=..., header: bool=..., index: bool=..., na_rep: str=..., name: bool=..., float_format: Optional[str]=..., dtype: bool=..., max_rows: Optional[int]=..., min_rows: Optional[int]=...) -> None: ...
    def to_string(self) -> str: ...

class TextAdjustment:
    encoding: Any = ...
    def __init__(self) -> None: ...
    def len(self, text: str) -> int: ...
    def justify(self, texts: Any, max_len: int, mode: str=...) -> List[str]: ...
    def adjoin(self, space: int, *lists: Any, **kwargs: Any) -> str: ...

class EastAsianTextAdjustment(TextAdjustment):
    ambiguous_width: int = ...
    def __init__(self) -> None: ...
    def len(self, text: str) -> int: ...
    def justify(self, texts: Iterable[str], max_len: int, mode: str=...) -> List[str]: ...

class TableFormatter:
    show_dimensions: Union[bool, str]
    is_truncated: bool
    formatters: formatters_type
    columns: Index
    @property
    def should_show_dimensions(self) -> bool: ...
    def get_buffer(self, buf: Optional[FilePathOrBuffer[str]], encoding: Optional[str]=...) -> Any: ...
    def write_result(self, buf: IO[str]) -> None: ...
    def get_result(self, buf: Optional[FilePathOrBuffer[str]]=..., encoding: Optional[str]=...) -> Optional[str]: ...

class DataFrameFormatter(TableFormatter):
    __doc__: Any = ...
    frame: Any = ...
    show_index_names: Any = ...
    sparsify: Any = ...
    float_format: Any = ...
    formatters: Any = ...
    na_rep: Any = ...
    decimal: Any = ...
    col_space: Any = ...
    header: Any = ...
    index: Any = ...
    line_width: Any = ...
    max_rows: Any = ...
    min_rows: Any = ...
    max_cols: Any = ...
    max_rows_displayed: Any = ...
    show_dimensions: Any = ...
    table_id: Any = ...
    render_links: Any = ...
    justify: Any = ...
    bold_rows: Any = ...
    escape: Any = ...
    columns: Any = ...
    adj: Any = ...
    def __init__(self, frame: DataFrame, columns: Optional[Sequence[str]]=..., col_space: Optional[Union[str, int]]=..., header: Union[bool, Sequence[str]]=..., index: bool=..., na_rep: str=..., formatters: Optional[formatters_type]=..., justify: Optional[str]=..., float_format: Optional[float_format_type]=..., sparsify: Optional[bool]=..., index_names: bool=..., line_width: Optional[int]=..., max_rows: Optional[int]=..., min_rows: Optional[int]=..., max_cols: Optional[int]=..., show_dimensions: Union[bool, str]=..., decimal: str=..., table_id: Optional[str]=..., render_links: bool=..., bold_rows: bool=..., escape: bool=...) -> None: ...
    max_cols_adj: Any = ...
    def write_result(self, buf: IO[str]) -> None: ...
    def to_string(self, buf: Optional[FilePathOrBuffer[str]]=..., encoding: Optional[str]=...) -> Optional[str]: ...
    def to_latex(self, buf: Optional[FilePathOrBuffer[str]]=..., column_format: Optional[str]=..., longtable: bool=..., encoding: Optional[str]=..., multicolumn: bool=..., multicolumn_format: Optional[str]=..., multirow: bool=..., caption: Optional[str]=..., label: Optional[str]=...) -> Optional[str]: ...
    def to_html(self, buf: Optional[FilePathOrBuffer[str]]=..., encoding: Optional[str]=..., classes: Optional[Union[str, List[Any], Tuple[Any, ...]]]=..., notebook: bool=..., border: Optional[int]=...) -> Optional[str]: ...
    @property
    def has_index_names(self) -> bool: ...
    @property
    def has_column_names(self) -> bool: ...
    @property
    def show_row_idx_names(self) -> bool: ...
    @property
    def show_col_idx_names(self) -> bool: ...

def format_array(values: Any, formatter: Optional[Callable[..., Any]], float_format: Optional[float_format_type]=..., na_rep: str=..., digits: Optional[int]=..., space: Optional[Union[str, int]]=..., justify: str=..., decimal: str=..., leading_space: Optional[bool]=...) -> List[str]: ...

class GenericArrayFormatter:
    values: Any = ...
    digits: Any = ...
    na_rep: Any = ...
    space: Any = ...
    formatter: Any = ...
    float_format: Any = ...
    justify: Any = ...
    decimal: Any = ...
    quoting: Any = ...
    fixed_width: Any = ...
    leading_space: Any = ...
    def __init__(self, values: Any, digits: int=..., formatter: Optional[Callable[..., Any]]=..., na_rep: str=..., space: Union[str, int]=..., float_format: Optional[float_format_type]=..., justify: str=..., decimal: str=..., quoting: Optional[int]=..., fixed_width: bool=..., leading_space: Optional[bool]=...) -> None: ...
    def get_result(self) -> List[str]: ...

class FloatArrayFormatter(GenericArrayFormatter):
    fixed_width: bool = ...
    formatter: Any = ...
    float_format: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def get_result_as_array(self) -> np.ndarray: ...

class IntArrayFormatter(GenericArrayFormatter): ...

class Datetime64Formatter(GenericArrayFormatter):
    nat_rep: Any = ...
    date_format: Any = ...
    def __init__(self, values: Union[np.ndarray, Series, DatetimeIndex, DatetimeArray], nat_rep: str=..., date_format: None=..., **kwargs: Any) -> None: ...

class ExtensionArrayFormatter(GenericArrayFormatter): ...

def format_percentiles(percentiles: Union[np.ndarray, List[Union[int, float]], List[float], List[Union[str, float]]]) -> List[str]: ...

class Datetime64TZFormatter(Datetime64Formatter): ...

class Timedelta64Formatter(GenericArrayFormatter):
    nat_rep: Any = ...
    box: Any = ...
    def __init__(self, values: Union[np.ndarray, TimedeltaIndex], nat_rep: str=..., box: bool=..., **kwargs: Any) -> None: ...

class EngFormatter:
    ENG_PREFIXES: Any = ...
    accuracy: Any = ...
    use_eng_prefix: Any = ...
    def __init__(self, accuracy: Optional[int]=..., use_eng_prefix: bool=...) -> None: ...
    def __call__(self, num: Union[int, float]) -> str: ...

def set_eng_float_format(accuracy: int=..., use_eng_prefix: bool=...) -> None: ...
def get_level_lengths(levels: Any, sentinel: Union[bool, object, str]=...) -> List[Dict[int, int]]: ...
def buffer_put_lines(buf: IO[str], lines: List[str]) -> None: ...



from pandas.io.common import stringify_path as stringify_path
from typing import Any, Optional

def read_sas(filepath_or_buffer: Any, format: Optional[Any] = ..., index: Optional[Any] = ..., encoding: Optional[Any] = ..., chunksize: Optional[Any] = ..., iterator: bool = ...) -> Any: ...



from __future__ import annotations

import collections
import sys
from io import StringIO
from typing import Any, Callable, Hashable, IO, Optional, Iterable, Union, Mapping, Sequence, Type, TypeVar, Dict, Set, \
    overload

if sys.version_info >= (3, 8):
    from typing import Literal
else:
    from typing_extensions import Literal

import numpy as np

from pandas._typing import ArrayLike as ArrayLike, OneDimensionalAxisOption
from pandas._typing import Renamer, FrameOrSeries, Function, AxisOption, Frequency, Scalar, Dtype, \
    NoneNumpyCompatible, Level, \
    GroupByObject, GeneralDuplicatesKeepStrategy, Label, InterpolationMethod, CorrelationMethod, SearchSide, \
    SortKind, \
    AggregationFunction, JoinType, FillMethod, ErrorsStrategy, FillValue, ToReplace, ReplaceValue, \
    TimestampMethod, \
    ReplaceMethod, ValueKeyFunc, IndexKeyFunc
from pandas.core import base, generic
from pandas.core.arrays import ExtensionArray
from pandas.core.frame import DataFrame, KeepStrategy, TransformFunction
from pandas.core.groupby import generic as groupby_generic
from pandas.core.indexes.base import Index

SortValuesNaPosition = Literal['first', 'last']
MapNaNAction = Literal['ignore']

K = TypeVar('K')
V = TypeVar('V')

CoercibleIntoSeries = Union[Scalar, Dict[str, Scalar], Iterable[Scalar], ArrayLike]

class Series(base.IndexOpsMixin, generic.NDFrame):
    index: Index = ...
    # These are accessors and might need some work
    str: Any = ...
    dt: Any = ...
    cat: Any = ...
    sparse: Any = ...
    plot: Any = ...
    hist: Any = ...
    def __init__(self, data: Optional[Any] = ..., index: Optional[Any] = ..., dtype: Optional[Dtype] = ..., name: Optional[Any] = ..., copy: bool = ..., fastpath: bool = ...) -> None: ...
    def __len__(self) -> int: ...
    @overload
    def __getitem__(self, key: Label) -> Any: ...
    @overload
    def __getitem__(self, key: slice) -> DataFrame: ...
    def __setitem__(self, key: Label, value: Scalar) -> None: ...
    DotOther = Union['Series', DataFrame, ArrayLike]
    DotResult = Union[Scalar, 'Series', ArrayLike]
    def __matmul__(self, other: DotOther) -> DotResult: ...
    def __rmatmul__(self, other: DotOther) -> DotResult: ...
    def __le__(self, other: Union[Scalar, Series]) -> Series: ...
    def __lt__(self, other: Union[Scalar, Series]) -> Series: ...
    def __ge__(self, other: Union[Scalar, Series]) -> Series: ...
    def __gt__(self, other: Union[Scalar, Series]) -> Series: ...
    def __mul__(self, other: Union[Scalar, Series]) -> Series: ...
    def __add__(self, other: Union[Scalar, Series]) -> Series: ...
    def __sub__(self, other: Union[Scalar, Series]) -> Series: ...
    def __truediv__(self, other: Union[Scalar, Series]) -> Series: ...
    def __floordiv__(self, other: Union[Scalar, Series]) -> Series: ...
    def __mod__(self, other: Union[Scalar, Series]) -> Series: ...
    def __and__(self, other: Union[Scalar, Series]) -> Series: ...
    def __or__(self, other: Union[Scalar, Series]) -> Series: ...
    def __invert__(self) -> Series: ...
    @property
    def dtype(self) -> Dtype: ...
    @property
    def dtypes(self) -> Dtype: ...
    @property
    def name(self) -> Optional[Hashable]: ...
    @name.setter
    def name(self, value: Optional[Hashable]) -> None: ...
    @property
    def values(self) -> Any: ...
    @property
    def hasnans(self) -> bool: ...
    @property
    def array(self) -> ExtensionArray: ...
    def div(self,other: Union[Scalar, Series], axis: Optional[OneDimensionalAxisOption] = ...,level: Optional[Level] = ..., fill_value: Optional[Scalar] = ...,) -> Series: ...
    def rdiv(self,other: Union[Scalar, Series], axis: Optional[OneDimensionalAxisOption] = ...,level: Optional[Level] = ..., fill_value: Optional[Scalar] = ...,) -> Series: ...
    def ravel(self, order: str = ...) -> Any: ...
    def view(self, dtype: Optional[Dtype] = ...) -> Any: ...
    def __array_ufunc__(self, ufunc: Function, method: str, *inputs: Any, **kwargs: Any) -> Any: ...
    def __array__(self, dtype: Any = ...) -> np.ndarray: ...
    @property
    def axes(self) -> Any: ...
    def take(self, indices: Union[Iterable[int], np.ndarray[np.int_]], axis: AxisOption = ..., is_copy: bool = ..., **kwargs: Any) -> Series: ...
    def repeat(self, repeats: Union[int, Iterable[int]], axis: Optional[NoneNumpyCompatible] = ...) -> Series: ...
    @overload
    def reset_index(self, level: Optional[Level] = ..., drop: bool = ..., name: Optional[Any] = ..., *, inplace: Literal[False] = ...) -> FrameOrSeries: ...
    @overload
    def reset_index(self, level: Optional[Level] = ..., drop: bool = ..., name: Optional[Any] = ..., *, inplace: Literal[True]) -> None: ...
    def to_string(self, buf: Optional[StringIO] = ..., na_rep: Optional[str] = ..., float_format: Optional[Callable[[Any], Any]] = ..., header: bool = ..., index: bool = ..., length: bool = ..., dtype: bool = ..., name: bool = ..., max_rows: Optional[int] = ..., min_rows: Optional[int] = ...) -> Union[str, None]: ...
    def to_markdown(self, buf: Optional[IO[str]]=..., mode: Optional[str]=..., index: bool = ..., **kwargs: Any) -> Optional[str]: ...
    def items(self) -> Iterable[Any]: ...
    def iteritems(self) -> Iterable[Any]: ...
    def keys(self) -> Index: ...
    def to_dict(self, into: Type[Mapping[K,V]] = ...) -> Type[Mapping[K,V]]: ...
    def to_frame(self, name: Optional[Any] = ...) -> DataFrame: ...
    def groupby(self, by: Optional[GroupByObject] = ..., axis: AxisOption = ..., level: Optional[Union[Level, Iterable[Level]]] = ..., as_index: bool=..., sort: bool=..., group_keys: bool=..., squeeze: bool=..., observed: bool=..., dropna: bool=...) -> groupby_generic.SeriesGroupBy: ...
    def count(self, level: Optional[Level] = ...) -> Union[int, Series]: ...
    def mode(self, dropna: bool = ...) -> Series: ...
    def unique(self) -> ArrayLike: ...
    @overload
    def drop_duplicates(self, keep: GeneralDuplicatesKeepStrategy = ..., *, inplace: Literal[False] = ...) -> Series: ...
    @overload
    def drop_duplicates(self, keep: GeneralDuplicatesKeepStrategy = ..., *, inplace: Literal[True]) -> None: ...
    def duplicated(self, keep: GeneralDuplicatesKeepStrategy = ...) -> Series: ...
    def idxmin(self, axis: int = ..., skipna: bool = ..., *args: Any, **kwargs: Any) -> Label: ...
    def idxmax(self, axis: int = ..., skipna: bool = ..., *args: Any, **kwargs: Any) -> Label: ...
    def round(self, decimals: int = ..., *args: Any, **kwargs: Any) -> Series: ...
    def quantile(self, q: Union[float, ArrayLike] = ..., interpolation: InterpolationMethod = ...) -> Union[float, Series]: ...
    def corr(self, other: Series, method: CorrelationMethod = ..., min_periods: Optional[int] = ...) -> float: ...
    def cov(self, other: Series, min_periods: Optional[int] = ..., ddof: Optional[int] = ...) -> float: ...
    def diff(self, periods: int = ...) -> Series: ...
    def autocorr(self, lag: int = ...) -> float: ...
    def dot(self, other: DotOther) -> DotResult: ...
    def searchsorted(self, value: ArrayLike, side: SearchSide = ..., sorter: Optional[ArrayLike] = ...) -> Sequence[int]: ...   # type: ignore[override]
    def append(self, to_append: Union[Series, Iterable[Series]], ignore_index: bool = ..., verify_integrity: bool = ...) -> Series: ...
    def combine(self, other: Union[Series, Scalar], func: Callable[[Scalar, Scalar], Scalar], fill_value: Optional[Scalar] = ...) -> Series: ...
    def combine_first(self, other: Series) -> Series: ...
    def update(self, other: Union[Series, CoercibleIntoSeries]) -> None: ...
    @overload    # type: ignore[override]
    def sort_values(self, axis: OneDimensionalAxisOption = ..., ascending: bool = ..., kind: SortKind = ..., na_position: SortValuesNaPosition = ..., ignore_index: bool = ..., key: ValueKeyFunc = ..., *, inplace: Literal[False] = ...) -> Series: ...
    @overload
    def sort_values(self, axis: OneDimensionalAxisOption = ..., ascending: bool = ..., kind: SortKind = ..., na_position: SortValuesNaPosition = ..., ignore_index: bool = ..., key: ValueKeyFunc = ..., *, inplace: Literal[True]) -> None: ...
    @overload
    def sort_index(self, axis: int = ..., level: int = ..., ascending: Union[bool, Iterable[bool]] = ..., kind: SortKind = ..., na_position: SortValuesNaPosition = ..., sort_remaining: bool = ..., ignore_index: bool = ..., key: IndexKeyFunc = ..., *,  inplace: Literal[False] = ...) -> Series: ...
    @overload
    def sort_index(self, axis: int = ..., level: int = ..., ascending: Union[bool, Iterable[bool]] = ..., kind: SortKind = ..., na_position: SortValuesNaPosition = ..., sort_remaining: bool = ..., ignore_index: bool = ..., key: IndexKeyFunc = ..., *,  inplace: Literal[True]) -> None: ...
    def argsort(self, axis: OneDimensionalAxisOption = ..., kind: SortKind = ..., order: Optional[Any] = ...) -> Series: ...
    def nlargest(self, n: int = ..., keep: KeepStrategy = ...) -> Series: ...
    def nsmallest(self, n: int = ..., keep: KeepStrategy = ...) -> Series: ...
    def swaplevel(self, i: int = ..., j: int = ..., copy: bool = ...) -> Series: ...
    def reorder_levels(self, order: Iterable[int]) -> Series: ...
    def explode(self) -> Series: ...
    def unstack(self, level: Union[Level, Iterable[Level]] = ..., fill_value: Optional[Scalar] = ...) -> DataFrame: ...
    def map(self, arg: Union[Function, collections.abc.Mapping[Any, Any], Series], na_action: Optional[MapNaNAction] = ...) -> Series: ...
    def aggregate(self, func: AggregationFunction = ..., axis: OneDimensionalAxisOption = ..., *args: Any, **kwargs: Any) -> FrameOrSeries: ...
    def agg(self, func: AggregationFunction = ..., axis: OneDimensionalAxisOption = ..., *args: Any, **kwargs: Any) -> FrameOrSeries: ...
    def transform(self, func: TransformFunction, axis: OneDimensionalAxisOption = ..., *args: Any, **kwargs: Any) -> Series: ...
    def apply(self, func: Function, convert_dtype: bool = ..., args: Any = ..., **kwds: Any) -> FrameOrSeries: ...
    def align(self, other: FrameOrSeries, join: JoinType = ..., axis: Optional[OneDimensionalAxisOption] = ..., level: Optional[Level] = ..., copy: bool = ..., fill_value: Optional[Scalar] = ..., method: Optional[FillMethod] = ..., limit: Optional[int] = ..., fill_axis: OneDimensionalAxisOption = ..., broadcast_axis: Optional[OneDimensionalAxisOption] = ...) -> Series: ...  # type: ignore
    @overload  #type: ignore[override]
    def rename(self, index: Renamer = ..., *, axis: Optional[OneDimensionalAxisOption] = ..., copy: bool = ..., inplace: Literal[False] = ..., level: Optional[Level] = ..., errors: ErrorsStrategy = ...) -> Series: ...
    @overload
    def rename(self, index: Renamer = ..., *, axis: Optional[OneDimensionalAxisOption] = ..., copy: bool = ..., inplace: Literal[True], level: Optional[Level] = ..., errors: ErrorsStrategy = ...) -> None: ...
    def reindex(self, index: Optional[Union[Label, Iterable[Label]]] = ..., **kwargs: Any) -> FrameOrSeries: ... # type: ignore[override]
    @overload
    def drop(self, labels: Optional[Union[Label, Iterable[Label]]] = ..., axis: OneDimensionalAxisOption = ..., index: Optional[Union[Label, Iterable[Label]]]  = ..., columns: Optional[Union[Label, Iterable[Label]]]  = ..., level: Optional[Level] = ..., errors: ErrorsStrategy = ..., *, inplace: Literal[False] = ...) -> Series: ...
    @overload
    def drop(self, labels: Optional[Union[Label, Iterable[Label]]] = ..., axis: OneDimensionalAxisOption = ..., index: Optional[Union[Label, Iterable[Label]]]  = ..., columns: Optional[Union[Label, Iterable[Label]]]  = ..., level: Optional[Level] = ..., errors: ErrorsStrategy = ...,*, inplace: Literal[True]) -> None: ...
    @overload
    def fillna(self, value: FillValue = ..., method: FillMethod = ..., axis: OneDimensionalAxisOption = ..., limit: int = ..., downcast: Dict[Any, Dtype] = ..., *, inplace: Literal[False] = ...) -> Series: ...
    @overload
    def fillna(self, value: FillValue = ..., method: FillMethod = ..., axis: OneDimensionalAxisOption = ..., limit: int = ..., downcast: Dict[Any, Dtype] = ..., *, inplace: Literal[True]) -> None: ...
    @overload
    def replace(self, to_replace: Optional[ToReplace] = ..., value: Optional[ReplaceValue] = ..., limit: Optional[int] = ..., regex: bool = ..., method: ReplaceMethod = ..., *, inplace: Literal[False] = ...) -> Series: ...
    @overload
    def replace(self, to_replace: Optional[ToReplace] = ..., value: Optional[ReplaceValue] = ..., limit: Optional[int] = ..., regex: bool = ..., method: ReplaceMethod = ..., *, inplace: Literal[True]) -> None: ...
    def shift(self, periods: int = ..., freq: Optional[Frequency] = ..., axis: AxisOption = ..., fill_value: Scalar = ...) -> Series: ...
    def memory_usage(self, index: bool = ..., deep: bool = ...) -> int: ... # type: ignore[override]
    def isin(self, values: Union[Set[Scalar], Iterable[Scalar]]) -> Series: ...
    def between(self, left: Union[Scalar, Iterable[Scalar]], right: Union[Scalar, Iterable[Scalar]], inclusive: bool = ...) -> Series: ...
    def isna(self) -> Series: ...
    def isnull(self) -> Series: ...
    def notna(self) -> Series: ...
    def notnull(self) -> Series: ...
    @overload
    def dropna(self, axis: OneDimensionalAxisOption = ..., how: Optional[Any] = ..., *, inplace: Literal[False] = ...) -> Series: ...
    @overload
    def dropna(self, axis: OneDimensionalAxisOption = ..., how: Optional[Any] = ..., *, inplace: Literal[True]) -> None: ...
    def to_timestamp(self, freq: Optional[str] = ..., how: TimestampMethod = ..., copy: bool = ...) -> Series: ...
    def to_period(self, freq: Optional[str] = ..., copy: bool = ...) -> Series: ...
    def compare(self, other: Series, align_axis: AxisOption = ..., keep_shape: bool = ..., keep_equal: bool = ...) -> FrameOrSeries: ...


from pandas._libs.tslibs import Timestamp as Timestamp
from typing import Any, Optional

def ensure_scope(level: int, global_dict: Any = ..., local_dict: Any = ..., resolvers: Any = ..., target: Any = ..., **kwargs: Any) -> Scope: ...

class Scope:
    level: Any = ...
    scope: Any = ...
    target: Any = ...
    resolvers: Any = ...
    temps: Any = ...
    def __init__(self, level: Any, global_dict: Optional[Any] = ..., local_dict: Optional[Any] = ..., resolvers: Any = ..., target: Optional[Any] = ...) -> None: ...
    @property
    def has_resolvers(self) -> bool: ...
    def resolve(self, key: str, is_local: bool) -> Any: ...
    def swapkey(self, old_key: str, new_key: str, new_value: Any = ...) -> Any: ...
    def add_tmp(self, value: Any) -> str: ...
    @property
    def ntemps(self) -> int: ...
    @property
    def full_scope(self) -> Any: ...



from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame, ABCIndexClass as ABCIndexClass, ABCMultiIndex as ABCMultiIndex, ABCPeriodIndex as ABCPeriodIndex, ABCSeries as ABCSeries
from pandas.core.dtypes.missing import isna as isna, notna as notna
from pandas.errors import AbstractMethodError as AbstractMethodError
from pandas.io.formats.printing import pprint_thing as pprint_thing
from pandas.plotting._matplotlib.converter import register_pandas_matplotlib_converters as register_pandas_matplotlib_converters
from pandas.plotting._matplotlib.tools import format_date_labels as format_date_labels, table as table
from pandas.util._decorators import cache_readonly as cache_readonly
from typing import Any, Optional

class MPLPlot:
    orientation: Optional[str] = ...
    data: Any = ...
    by: Any = ...
    kind: Any = ...
    sort_columns: Any = ...
    subplots: Any = ...
    sharex: bool = ...
    sharey: Any = ...
    figsize: Any = ...
    layout: Any = ...
    xticks: Any = ...
    yticks: Any = ...
    xlim: Any = ...
    ylim: Any = ...
    title: Any = ...
    use_index: Any = ...
    fontsize: Any = ...
    rot: Any = ...
    grid: Any = ...
    legend: Any = ...
    legend_handles: Any = ...
    legend_labels: Any = ...
    ax: Any = ...
    fig: Any = ...
    axes: Any = ...
    errors: Any = ...
    secondary_y: Any = ...
    colormap: Any = ...
    table: Any = ...
    include_bool: Any = ...
    kwds: Any = ...
    def __init__(self, data: Any, kind: Optional[Any] = ..., by: Optional[Any] = ..., subplots: bool = ..., sharex: Optional[Any] = ..., sharey: bool = ..., use_index: bool = ..., figsize: Optional[Any] = ..., grid: Optional[Any] = ..., legend: bool = ..., rot: Optional[Any] = ..., ax: Optional[Any] = ..., fig: Optional[Any] = ..., title: Optional[Any] = ..., xlim: Optional[Any] = ..., ylim: Optional[Any] = ..., xticks: Optional[Any] = ..., yticks: Optional[Any] = ..., sort_columns: bool = ..., fontsize: Optional[Any] = ..., secondary_y: bool = ..., colormap: Optional[Any] = ..., table: bool = ..., layout: Optional[Any] = ..., include_bool: bool = ..., **kwds: Any) -> None: ...
    @property
    def nseries(self) -> Any: ...
    def draw(self) -> None: ...
    def generate(self) -> None: ...
    @property
    def result(self) -> Any: ...
    @property
    def legend_title(self) -> Any: ...
    def plt(self) -> Any: ...
    @classmethod
    def get_default_ax(cls, ax: Any) -> None: ...
    def on_right(self, i: Any) -> Any: ...

class PlanePlot(MPLPlot):
    x: Any = ...
    y: Any = ...
    def __init__(self, data: Any, x: Any, y: Any, **kwargs: Any) -> None: ...
    @property
    def nseries(self) -> Any: ...

class ScatterPlot(PlanePlot):
    c: Any = ...
    def __init__(self, data: Any, x: Any, y: Any, s: Optional[Any] = ..., c: Optional[Any] = ..., **kwargs: Any) -> None: ...

class HexBinPlot(PlanePlot):
    C: Any = ...
    def __init__(self, data: Any, x: Any, y: Any, C: Optional[Any] = ..., **kwargs: Any) -> None: ...

class LinePlot(MPLPlot):
    orientation: str = ...
    data: Any = ...
    x_compat: Any = ...
    def __init__(self, data: Any, **kwargs: Any) -> None: ...

class AreaPlot(LinePlot):
    def __init__(self, data: Any, **kwargs: Any) -> None: ...

class BarPlot(MPLPlot):
    orientation: str = ...
    bar_width: Any = ...
    tick_pos: Any = ...
    bottom: Any = ...
    left: Any = ...
    log: Any = ...
    tickoffset: Any = ...
    lim_offset: Any = ...
    ax_pos: Any = ...
    def __init__(self, data: Any, **kwargs: Any) -> None: ...

class BarhPlot(BarPlot):
    orientation: str = ...

class PiePlot(MPLPlot):
    def __init__(self, data: Any, kind: Optional[Any] = ..., **kwargs: Any) -> None: ...



from pandas._config import get_option as get_option
from pandas.core.dtypes.inference import is_sequence as is_sequence
from typing import Any, Callable, Iterable, List, Mapping, Optional, Sequence, Tuple, Union

EscapeChars = Union[Mapping[str, str], Iterable[str]]

def adjoin(space: int, *lists: List[str], **kwargs: Any) -> str: ...
def justify(texts: Iterable[str], max_len: int, mode: str=...) -> List[str]: ...
def pprint_thing(thing: Any, _nest_lvl: int=..., escape_chars: Optional[EscapeChars]=..., default_escapes: bool=..., quote_strings: bool=..., max_seq_items: Optional[int]=...) -> str: ...
def pprint_thing_encoded(object: Any, encoding: str=..., errors: str=...) -> bytes: ...

default_pprint: Any

def format_object_summary(obj: Any, formatter: Callable[..., Any], is_justify: bool=..., name: Optional[str]=..., indent_for_name: bool=..., line_break_each_value: bool=...) -> str: ...
def format_object_attrs(obj: Sequence[Any], include_dtype: bool=...) -> List[Tuple[str, Union[str, int]]]: ...



import numpy as np
from pandas._config import get_option as get_option
from pandas._libs.interval import Interval as Interval, IntervalMixin as IntervalMixin, IntervalTree as IntervalTree
from pandas._typing import AnyArrayLike as AnyArrayLike
from pandas.core.algorithms import take_1d as take_1d
from pandas.core.arrays.interval import IntervalArray as IntervalArray
from pandas.core.dtypes.generic import ABCSeries as ABCSeries
from pandas.core.dtypes.missing import isna as isna
from pandas.core.indexes.base import Index as Index, InvalidIndexError as InvalidIndexError
from pandas.core.indexes.datetimes import DatetimeIndex as DatetimeIndex, date_range as date_range
from pandas.core.indexes.extension import ExtensionIndex as ExtensionIndex, inherit_names as inherit_names
from pandas.core.indexes.multi import MultiIndex as MultiIndex
from pandas.core.indexes.timedeltas import TimedeltaIndex as TimedeltaIndex, timedelta_range as timedelta_range
from pandas.core.ops import get_op_result_name as get_op_result_name
from pandas.tseries.frequencies import to_offset as to_offset
from pandas.tseries.offsets import DateOffset as DateOffset
from pandas.util._decorators import Appender as Appender, Substitution as Substitution, cache_readonly as cache_readonly
from pandas.util._exceptions import rewrite_exception as rewrite_exception
from typing import Any, Optional, Tuple, Union

class SetopCheck:
    op_name: Any = ...
    def __init__(self, op_name: Any) -> None: ...
    def __call__(self, setop: Any) -> Any: ...

class IntervalIndex(IntervalMixin, ExtensionIndex):
    def __new__(cls: Any, data: Any, closed: Any = ..., dtype: Any = ..., copy: bool=..., name: Any = ..., verify_integrity: bool=...) -> Any: ...
    @classmethod
    def from_breaks(cls: Any, breaks: Any, closed: str=..., name: Any = ..., copy: bool=..., dtype: Any = ...) -> Any: ...
    @classmethod
    def from_arrays(cls: Any, left: Any, right: Any, closed: str=..., name: Any = ..., copy: bool=..., dtype: Any = ...) -> Any: ...
    @classmethod
    def from_tuples(cls: Any, data: Any, closed: str=..., name: Any = ..., copy: bool=..., dtype: Any = ...) -> Any: ...
    def __contains__(self, key: Any) -> bool: ...
    def values(self) -> Any: ...
    def __array_wrap__(self, result: Any, context: Optional[Any] = ...) -> Any: ...
    def __reduce__(self) -> Any: ...
    def astype(self, dtype: Any, copy: bool = ...) -> Any: ...
    @property
    def inferred_type(self) -> str: ...
    def memory_usage(self, deep: bool=...) -> int: ...
    def is_monotonic(self) -> bool: ...
    def is_monotonic_increasing(self) -> bool: ...
    def is_monotonic_decreasing(self) -> bool: ...
    def is_unique(self) -> Any: ...
    @property
    def is_overlapping(self) -> Any: ...
    def get_loc(self, key: Any, method: Optional[str]=..., tolerance: Any = ...) -> Union[int, slice, np.ndarray]: ...
    def get_indexer(self, target: AnyArrayLike, method: Optional[str]=..., limit: Optional[int]=..., tolerance: Optional[Any]=...) -> np.ndarray: ...
    def get_indexer_non_unique(self, target: AnyArrayLike) -> Tuple[np.ndarray, np.ndarray]: ...
    def get_indexer_for(self, target: AnyArrayLike, **kwargs: Any) -> np.ndarray: ...
    def get_value(self, series: ABCSeries, key: Any) -> Any: ...
    def where(self, cond: Any, other: Optional[Any] = ...) -> Any: ...
    def delete(self, loc: Any) -> Any: ...
    def insert(self, loc: Any, item: Any) -> Any: ...
    def take(self, indices: Any, axis: int = ..., allow_fill: bool = ..., fill_value: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def __getitem__(self, value: Any) -> Any: ...
    def argsort(self, *args: Any, **kwargs: Any) -> Any: ...
    def equals(self, other: Any) -> bool: ...
    def intersection(self, other: IntervalIndex, sort: bool=...) -> IntervalIndex: ...
    @property
    def is_all_dates(self) -> bool: ...
    union: Any = ...
    difference: Any = ...
    symmetric_difference: Any = ...
    def __lt__(self, other: Any) -> Any: ...
    def __le__(self, other: Any) -> Any: ...
    def __gt__(self, other: Any) -> Any: ...
    def __ge__(self, other: Any) -> Any: ...

def interval_range(start: Optional[Any] = ..., end: Optional[Any] = ..., periods: Optional[Any] = ..., freq: Optional[Any] = ..., name: Optional[Any] = ..., closed: str = ...) -> Any: ...



from pandas._typing import FrameOrSeries as FrameOrSeries
from pandas.core.base import PandasObject as PandasObject
from pandas.core.computation.common import result_type_many as result_type_many
from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame, ABCSeries as ABCSeries
from pandas.errors import PerformanceWarning as PerformanceWarning
from typing import Any

def align_terms(terms: Any) -> Any: ...
def reconstruct_object(typ: Any, obj: Any, axes: Any, dtype: Any) -> Any: ...



from pandas.core.arrays import Categorical as Categorical

from pandas.core.dtypes.generic import ABCMultiIndex as ABCMultiIndex
from pandas.core.dtypes.missing import notna as notna
from pandas.core.frame import DataFrame as DataFrame
from pandas.core.indexes.base import Index as Index
from pandas.core.reshape.concat import concat as concat
from pandas.core.tools.numeric import to_numeric as to_numeric
from pandas.util._decorators import Appender as Appender, deprecate_kwarg as deprecate_kwarg
from typing import Any

def melt(frame: DataFrame, id_vars: Any = ..., value_vars: Any = ..., var_name: Any = ..., value_name: Any = ..., col_level: Any = ..., ignore_index: bool = ...) -> DataFrame: ...
def lreshape(data: DataFrame, groups: Any, dropna: bool=..., label: Any = ...) -> DataFrame: ...
def wide_to_long(df: DataFrame, stubnames: Any, i: Any, j: Any, sep: str=..., suffix: str=...) -> DataFrame: ...



import numpy as np
from pandas._config import get_option as get_option
from pandas._libs import index as libindex
from pandas._typing import GeneralDuplicatesKeepStrategy
from pandas.core.arrays import Categorical as Categorical
from pandas.core.arrays.categorical import factorize_from_iterables as factorize_from_iterables
from pandas.core.dtypes.cast import coerce_indexer_dtype as coerce_indexer_dtype
from pandas.core.dtypes.common import ensure_int64 as ensure_int64, ensure_platform_int as ensure_platform_int, is_categorical_dtype as is_categorical_dtype, is_hashable as is_hashable, is_integer as is_integer, is_iterator as is_iterator, is_list_like as is_list_like, is_object_dtype as is_object_dtype, is_scalar as is_scalar, pandas_dtype as pandas_dtype
from pandas.core.dtypes.dtypes import ExtensionDtype as ExtensionDtype
from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame
from pandas.core.dtypes.missing import array_equivalent as array_equivalent, isna as isna
from pandas.core.indexes.base import Index as Index, InvalidIndexError as InvalidIndexError, ensure_index as ensure_index
from pandas.core.indexes.frozen import FrozenList as FrozenList
from pandas.core.sorting import get_group_index as get_group_index, indexer_from_factorized as indexer_from_factorized, lexsort_indexer as lexsort_indexer
from pandas.core.util.hashing import hash_tuple as hash_tuple, hash_tuples as hash_tuples
from pandas.errors import PerformanceWarning as PerformanceWarning, UnsortedIndexError as UnsortedIndexError
from pandas.io.formats.printing import format_object_attrs as format_object_attrs, format_object_summary as format_object_summary, pprint_thing as pprint_thing
from pandas.util._decorators import Appender as Appender, cache_readonly as cache_readonly
from typing import Any, Hashable, Optional, Sequence, Union

class MultiIndexUIntEngine(libindex.BaseMultiIndexCodesEngine, libindex.UInt64Engine): ...  # type: ignore
class MultiIndexPyIntEngine(libindex.BaseMultiIndexCodesEngine, libindex.ObjectEngine): ... # type: ignore

class MultiIndex(Index):
    rename: Any = ...
    def __new__(cls: Any, levels: Any = ..., codes: Any = ..., sortorder: Any = ..., names: Any = ..., dtype: Any = ..., copy: Any = ..., name: Any = ..., verify_integrity: bool=..., _set_identity: bool=...) -> Any: ...
    @classmethod
    def from_arrays(cls, arrays: Any, sortorder: Optional[Any] = ..., names: Any = ...) -> Any: ...
    @classmethod
    def from_tuples(cls, tuples: Any, sortorder: Optional[Any] = ..., names: Optional[Any] = ...) -> Any: ...
    @classmethod
    def from_product(cls, iterables: Any, sortorder: Optional[Any] = ..., names: Any = ...) -> Any: ...
    @classmethod
    def from_frame(cls, df: Any, sortorder: Optional[Any] = ..., names: Optional[Any] = ...) -> Any: ...
    @property
    def shape(self) -> Any: ...
    @property
    def array(self) -> None: ...
    def levels(self) -> Any: ...
    def set_levels(self, levels: Any, level: Optional[Any] = ..., inplace: bool = ..., verify_integrity: bool = ...) -> Any: ...
    @property
    def codes(self) -> Any: ...
    def set_codes(self, codes: Any, level: Optional[Any] = ..., inplace: bool = ..., verify_integrity: bool = ...) -> Any: ...
    def copy(self, names: Optional[Any] = ..., dtype: Optional[Any] = ..., levels: Optional[Any] = ..., codes: Optional[Any] = ..., deep: bool = ..., _set_identity: bool = ..., **kwargs: Any) -> Any: ... # type: ignore
    def __array__(self, dtype: Any = ...) -> np.ndarray: ...
    def view(self, cls: Optional[Any] = ...) -> Any: ...
    def __contains__(self, key: Any) -> bool: ...
    def dtype(self) -> np.dtype: ...
    def memory_usage(self, deep: bool=...) -> int: ...
    def nbytes(self) -> int: ...
    def format(self, space: int = ..., sparsify: Optional[Any] = ..., adjoin: bool = ..., names: bool = ..., na_rep: Optional[Any] = ..., formatter: Optional[Any] = ...) -> Any: ...   # type: ignore
    def __len__(self) -> int: ...
    names: Any = ...
    def inferred_type(self) -> str: ...
    @property
    def values(self) -> Any: ...
    def is_monotonic_increasing(self) -> bool: ...
    def is_monotonic_decreasing(self) -> bool: ...
    def duplicated(self, keep: GeneralDuplicatesKeepStrategy = ...) -> Any: ...
    def fillna(self, value: Optional[Any] = ..., downcast: Optional[Any] = ...) -> None: ...
    def dropna(self, how: str = ...) -> Any: ...
    def get_value(self, series: Any, key: Any) -> Any: ...
    def get_level_values(self, level: Any) -> Any: ...
    def unique(self, level: Optional[Any] = ...) -> Any: ...
    def to_frame(self, index: bool = ..., name: Optional[Any] = ...) -> Any: ...
    def to_flat_index(self) -> Any: ...
    @property
    def is_all_dates(self) -> bool: ...
    def is_lexsorted(self) -> bool: ...
    def lexsort_depth(self) -> Any: ...
    def remove_unused_levels(self) -> Any: ...
    @property
    def nlevels(self) -> int: ...
    @property
    def levshape(self) -> Any: ...
    def __reduce__(self) -> Any: ...
    def __getitem__(self, key: Any) -> Any: ...
    def take(self, indices: Any, axis: int = ..., allow_fill: bool = ..., fill_value: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def append(self, other: Any) -> Any: ...
    def argsort(self, *args: Any, **kwargs: Any) -> Any: ...
    def repeat(self, repeats: Any, axis: Optional[Any] = ...) -> Any: ...
    def where(self, cond: Any, other: Optional[Any] = ...) -> None: ...
    def drop(self, codes: Any, level: Optional[Any] = ..., errors: str = ...) -> Any: ...   # type: ignore
    def swaplevel(self, i: int = ..., j: int = ...) -> Any: ...
    def reorder_levels(self, order: Any) -> Any: ...
    def sortlevel(self, level: int = ..., ascending: bool = ..., sort_remaining: bool = ...) -> Any: ...
    def get_indexer(self, target: Any, method: Optional[Any] = ..., limit: Optional[Any] = ..., tolerance: Optional[Any] = ...) -> Any: ...
    def get_indexer_non_unique(self, target: Any) -> Any: ...
    def reindex(self, target: Any, method: Optional[Any] = ..., level: Optional[Any] = ..., limit: Optional[Any] = ..., tolerance: Optional[Any] = ...) -> Any: ...
    def get_slice_bound(self, label: Union[Hashable, Sequence[Hashable]], side: str, kind: str) -> int: ...
    def slice_locs(self, start: Optional[Any] = ..., end: Optional[Any] = ..., step: Optional[Any] = ..., kind: Optional[Any] = ...) -> Any: ...
    def get_loc(self, key: Any, method: Optional[Any] = ...) -> Any: ...    # type: ignore
    def get_loc_level(self, key: Any, level: Any = ..., drop_level: bool=...) -> Any: ...
    def get_locs(self, seq: Any) -> Any: ...
    def truncate(self, before: Optional[Any] = ..., after: Optional[Any] = ...) -> Any: ...
    def equals(self, other: Any) -> bool: ...
    def equal_levels(self, other: Any) -> Any: ...
    def union(self, other: Any, sort: Optional[Any] = ...) -> Any: ...
    def intersection(self, other: Any, sort: bool = ...) -> Any: ...
    def difference(self, other: Any, sort: Optional[Any] = ...) -> Any: ...
    def astype(self, dtype: Any, copy: bool = ...) -> Any: ...
    def insert(self, loc: Any, item: Any) -> Any: ...
    def delete(self, loc: Any) -> Any: ...
    def isin(self, values: Any, level: Optional[Any] = ...) -> Any: ...

def maybe_droplevels(index: Any, key: Any) -> Any: ...



from pandas.core.arrays import Categorical as Categorical, DatetimeArray as DatetimeArray, ExtensionArray as ExtensionArray, PandasArray as PandasArray, PandasDtype as PandasDtype, TimedeltaArray as TimedeltaArray
from pandas.core.base import PandasObject as PandasObject
from pandas.core.dtypes.cast import astype_nansafe as astype_nansafe, find_common_type as find_common_type, infer_dtype_from as infer_dtype_from, infer_dtype_from_scalar as infer_dtype_from_scalar, maybe_downcast_numeric as maybe_downcast_numeric, maybe_downcast_to_dtype as maybe_downcast_to_dtype, maybe_infer_dtype_type as maybe_infer_dtype_type, maybe_promote as maybe_promote, maybe_upcast as maybe_upcast, soft_convert_objects as soft_convert_objects
from pandas.core.dtypes.common import ensure_platform_int as ensure_platform_int, is_bool_dtype as is_bool_dtype, is_categorical as is_categorical, is_categorical_dtype as is_categorical_dtype, is_datetime64_dtype as is_datetime64_dtype, is_datetime64tz_dtype as is_datetime64tz_dtype, is_dtype_equal as is_dtype_equal, is_extension_array_dtype as is_extension_array_dtype, is_float_dtype as is_float_dtype, is_integer as is_integer, is_integer_dtype as is_integer_dtype, is_interval_dtype as is_interval_dtype, is_list_like as is_list_like, is_object_dtype as is_object_dtype, is_period_dtype as is_period_dtype, is_re as is_re, is_re_compilable as is_re_compilable, is_sparse as is_sparse, is_timedelta64_dtype as is_timedelta64_dtype, pandas_dtype as pandas_dtype
from pandas.core.dtypes.concat import concat_categorical as concat_categorical, concat_datetime as concat_datetime
from pandas.core.dtypes.dtypes import CategoricalDtype as CategoricalDtype, ExtensionDtype as ExtensionDtype
from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame, ABCExtensionArray as ABCExtensionArray, ABCPandasArray as ABCPandasArray, ABCSeries as ABCSeries
from pandas.core.dtypes.missing import array_equivalent as array_equivalent, is_valid_nat_for_dtype as is_valid_nat_for_dtype, isna as isna
from pandas.core.indexers import check_setitem_lengths as check_setitem_lengths, is_empty_indexer as is_empty_indexer, is_scalar_indexer as is_scalar_indexer
from pandas.core.nanops import nanpercentile as nanpercentile
from pandas.io.formats.printing import pprint_thing as pprint_thing
from pandas.util._validators import validate_bool_kwarg as validate_bool_kwarg
from typing import Any, List, Optional

class Block(PandasObject):
    is_numeric: bool = ...
    is_float: bool = ...
    is_integer: bool = ...
    is_complex: bool = ...
    is_datetime: bool = ...
    is_datetimetz: bool = ...
    is_timedelta: bool = ...
    is_bool: bool = ...
    is_object: bool = ...
    is_categorical: bool = ...
    is_extension: bool = ...
    ndim: Any = ...
    values: Any = ...
    def __init__(self, values: Any, placement: Any, ndim: Optional[Any] = ...) -> None: ...
    @property
    def is_view(self) -> Any: ...
    @property
    def is_datelike(self) -> Any: ...
    def is_categorical_astype(self, dtype: Any) -> Any: ...
    def external_values(self, dtype: Optional[Any] = ...) -> Any: ...
    def internal_values(self, dtype: Optional[Any] = ...) -> Any: ...
    def array_values(self) -> ExtensionArray: ...
    def get_values(self, dtype: Optional[Any] = ...) -> Any: ...
    def get_block_values(self, dtype: Optional[Any] = ...) -> Any: ...
    def to_dense(self) -> Any: ...
    @property
    def fill_value(self) -> Any: ...
    @property
    def mgr_locs(self) -> Any: ...
    @mgr_locs.setter
    def mgr_locs(self, new_mgr_locs: Any) -> None: ...
    @property
    def array_dtype(self) -> Any: ...
    def make_block(self, values: Any, placement: Any = ...) -> Block: ...
    def make_block_same_class(self, values: Any, placement: Optional[Any] = ..., ndim: Optional[Any] = ...) -> Any: ...
    def __len__(self) -> int: ...
    def getitem_block(self, slicer: Any, new_mgr_locs: Optional[Any] = ...) -> Any: ...
    @property
    def shape(self) -> Any: ...
    @property
    def dtype(self) -> Any: ...
    @property
    def ftype(self) -> Any: ...
    def merge(self, other: Any) -> Any: ...
    def concat_same_type(self, to_concat: Any, placement: Optional[Any] = ...) -> Any: ...
    def iget(self, i: Any) -> Any: ...
    def set(self, locs: Any, values: Any) -> None: ...
    def delete(self, loc: Any) -> None: ...
    def apply(self, func: Any, **kwargs: Any) -> Any: ...
    def fillna(self, value: Any, limit: Optional[Any] = ..., inplace: bool = ..., downcast: Optional[Any] = ...) -> Any: ...
    def split_and_operate(self, mask: Any, f: Any, inplace: bool) -> Any: ...
    def downcast(self, dtypes: Optional[Any] = ...) -> Any: ...
    def astype(self, dtype: Any, copy: bool=..., errors: str=...) -> Any: ...
    def convert(self, copy: bool=..., datetime: bool=..., numeric: bool=..., timedelta: bool=..., coerce: bool=...) -> Any: ...
    def to_native_types(self, slicer: Optional[Any] = ..., na_rep: str = ..., quoting: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def copy(self, deep: bool = ...) -> Any: ...
    def replace(self, to_replace: Any, value: Any, inplace: bool = ..., filter: Optional[Any] = ..., regex: bool = ..., convert: bool = ...) -> Any: ...
    def setitem(self, indexer: Any, value: Any) -> Any: ...
    def putmask(self, mask: Any, new: Any, align: bool = ..., inplace: bool = ..., axis: int = ..., transpose: bool = ...) -> Any: ...
    def coerce_to_target_dtype(self, other: Any) -> Any: ...
    def interpolate(self, method: str = ..., axis: int = ..., index: Optional[Any] = ..., values: Optional[Any] = ..., inplace: bool = ..., limit: Optional[Any] = ..., limit_direction: str = ..., limit_area: Optional[Any] = ..., fill_value: Optional[Any] = ..., coerce: bool = ..., downcast: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def take_nd(self, indexer: Any, axis: Any, new_mgr_locs: Optional[Any] = ..., fill_tuple: Optional[Any] = ...) -> Any: ...
    def diff(self, n: int, axis: int=...) -> List[Block]: ...
    def shift(self, periods: Any, axis: int = ..., fill_value: Optional[Any] = ...) -> Any: ...
    def where(self, other: Any, cond: Any, align: Any = ..., errors: Any = ..., try_cast: bool=..., axis: int=...) -> List[Block]: ...
    def equals(self, other: Any) -> bool: ...
    def quantile(self, qs: Any, interpolation: str = ..., axis: int = ...) -> Any: ...

class NonConsolidatableMixIn:
    def __init__(self, values: Any, placement: Any, ndim: Optional[Any] = ...) -> None: ...
    @property
    def shape(self) -> Any: ...
    def iget(self, col: Any) -> Any: ...
    def should_store(self, value: Any) -> Any: ...
    values: Any = ...
    def set(self, locs: Any, values: Any, check: bool = ...) -> None: ...
    def putmask(self, mask: Any, new: Any, align: bool = ..., inplace: bool = ..., axis: int = ..., transpose: bool = ...) -> Any: ...

class ExtensionBlock(NonConsolidatableMixIn, Block):
    is_extension: bool = ...
    def __init__(self, values: Any, placement: Any, ndim: Optional[Any] = ...) -> None: ...
    @property
    def fill_value(self) -> Any: ...
    @property
    def is_view(self) -> bool: ...
    @property
    def is_numeric(self) -> bool: ...   # type: ignore
    def setitem(self, indexer: Any, value: Any) -> Any: ...
    def get_values(self, dtype: Optional[Any] = ...) -> Any: ...
    def array_values(self) -> ExtensionArray: ...
    def to_dense(self) -> Any: ...
    def to_native_types(self, slicer: Optional[Any] = ..., na_rep: str = ..., quoting: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def take_nd(self, indexer: Any, axis: int = ..., new_mgr_locs: Optional[Any] = ..., fill_tuple: Optional[Any] = ...) -> Any: ...
    def concat_same_type(self, to_concat: Any, placement: Optional[Any] = ...) -> Any: ...
    def fillna(self, value: Any, limit: Optional[Any] = ..., inplace: bool = ..., downcast: Optional[Any] = ...) -> Any: ...
    def interpolate(self, method: str = ..., axis: int = ..., inplace: bool = ..., limit: Optional[Any] = ..., fill_value: Optional[Any] = ..., **kwargs: Any) -> Any: ...  # type: ignore
    def diff(self, n: int, axis: int=...) -> List[Block]: ...
    def shift(self, periods: int, axis: int=..., fill_value: Any = ...) -> List[ExtensionBlock]: ...
    def where(self, other: Any, cond: Any, align: Any = ..., errors: Any = ..., try_cast: bool=..., axis: int=...) -> List[Block]: ...

class ObjectValuesExtensionBlock(ExtensionBlock):
    def external_values(self, dtype: Optional[Any] = ...) -> Any: ...

class NumericBlock(Block):
    is_numeric: bool = ...

class FloatOrComplexBlock(NumericBlock):
    def equals(self, other: Any) -> bool: ...

class FloatBlock(FloatOrComplexBlock):
    is_float: bool = ...
    def to_native_types(self, slicer: Optional[Any] = ..., na_rep: str = ..., float_format: Optional[Any] = ..., decimal: str = ..., quoting: Optional[Any] = ..., **kwargs: Any) -> Any: ...   # type: ignore
    def should_store(self, value: Any) -> Any: ...

class ComplexBlock(FloatOrComplexBlock):
    is_complex: bool = ...
    def should_store(self, value: Any) -> Any: ...

class IntBlock(NumericBlock):
    is_integer: bool = ...
    def should_store(self, value: Any) -> Any: ...

class DatetimeLikeBlockMixin:
    @property
    def fill_value(self) -> Any: ...
    def get_values(self, dtype: Optional[Any] = ...) -> Any: ...
    def iget(self, key: Any) -> Any: ...
    def shift(self, periods: Any, axis: int = ..., fill_value: Optional[Any] = ...) -> Any: ...

class DatetimeBlock(DatetimeLikeBlockMixin, Block):
    is_datetime: bool = ...
    def __init__(self, values: Any, placement: Any, ndim: Optional[Any] = ...) -> None: ...
    def astype(self, dtype: Any, copy: bool=..., errors: str=...) -> Any: ...
    def to_native_types(self, slicer: Optional[Any] = ..., na_rep: Optional[Any] = ..., date_format: Optional[Any] = ..., quoting: Optional[Any] = ..., **kwargs: Any) -> Any: ...  # type: ignore
    def should_store(self, value: Any) -> Any: ...
    def set(self, locs: Any, values: Any) -> None: ...
    def external_values(self) -> Any: ...   # type: ignore
    def array_values(self) -> ExtensionArray: ...

class DatetimeTZBlock(ExtensionBlock, DatetimeBlock):
    is_datetimetz: bool = ...
    is_extension: bool = ...
    to_native_types: Any = ...
    fill_value: Any = ...
    @property
    def is_view(self) -> Any: ...
    def get_values(self, dtype: Optional[Any] = ...) -> Any: ...
    def to_dense(self) -> Any: ...
    def diff(self, n: int, axis: int=...) -> List[Block]: ...
    def concat_same_type(self, to_concat: Any, placement: Optional[Any] = ...) -> Any: ...
    def fillna(self, value: Any, limit: Optional[Any] = ..., inplace: bool = ..., downcast: Optional[Any] = ...) -> Any: ...
    def setitem(self, indexer: Any, value: Any) -> Any: ...
    def equals(self, other: Any) -> bool: ...
    def quantile(self, qs: Any, interpolation: str = ..., axis: int = ...) -> Any: ...

class TimeDeltaBlock(DatetimeLikeBlockMixin, IntBlock):
    is_timedelta: bool = ...
    is_numeric: bool = ...
    fill_value: Any = ...
    def __init__(self, values: Any, placement: Any, ndim: Optional[Any] = ...) -> None: ...
    def fillna(self, value: Any, **kwargs: Any) -> Any: ... # type: ignore
    def should_store(self, value: Any) -> Any: ...
    def to_native_types(self, slicer: Optional[Any] = ..., na_rep: Optional[Any] = ..., quoting: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def external_values(self, dtype: Optional[Any] = ...) -> Any: ...
    def array_values(self) -> ExtensionArray: ...

class BoolBlock(NumericBlock):
    is_bool: bool = ...
    def should_store(self, value: Any) -> Any: ...
    def replace(self, to_replace: Any, value: Any, inplace: bool = ..., filter: Optional[Any] = ..., regex: bool = ..., convert: bool = ...) -> Any: ...

class ObjectBlock(Block):
    is_object: bool = ...
    def __init__(self, values: Any, placement: Optional[Any] = ..., ndim: int = ...) -> None: ...
    @property
    def is_bool(self) -> Any: ...   # type: ignore
    def convert(self, copy: bool=..., datetime: bool=..., numeric: bool=..., timedelta: bool=..., coerce: bool=...) -> Any: ...
    def should_store(self, value: Any) -> Any: ...
    def replace(self, to_replace: Any, value: Any, inplace: bool = ..., filter: Optional[Any] = ..., regex: bool = ..., convert: bool = ...) -> Any: ...

class CategoricalBlock(ExtensionBlock):
    is_categorical: bool = ...
    def __init__(self, values: Any, placement: Any, ndim: Optional[Any] = ...) -> None: ...
    @property
    def array_dtype(self) -> Any: ...
    def to_dense(self) -> Any: ...
    def to_native_types(self, slicer: Optional[Any] = ..., na_rep: str = ..., quoting: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def concat_same_type(self, to_concat: Any, placement: Optional[Any] = ...) -> Any: ...
    def replace(self, to_replace: Any, value: Any, inplace: bool=..., filter: Any = ..., regex: bool=..., convert: bool=...) -> Any: ...

def get_block_type(values: Any, dtype: Optional[Any] = ...) -> Any: ...
def make_block(values: Any, placement: Any, klass: Optional[Any] = ..., ndim: Optional[Any] = ..., dtype: Optional[Any] = ...) -> Any: ...

def _block_shape(values: Any, ndim: int, shape: Any) -> Any: ...
def _safe_reshape(arr: Any, new_shape: Any) -> Any: ...


from pandas.core.arrays import ExtensionArray as ExtensionArray
from pandas.core.dtypes.common import ensure_platform_int as ensure_platform_int, is_dtype_equal as is_dtype_equal, is_object_dtype as is_object_dtype
from pandas.core.dtypes.generic import ABCSeries as ABCSeries
from pandas.core.indexers import deprecate_ndim_indexing as deprecate_ndim_indexing
from pandas.core.indexes.base import Index as Index
from pandas.core.ops import get_op_result_name as get_op_result_name
from pandas.util._decorators import Appender as Appender, cache_readonly as cache_readonly
from typing import Any, List, Optional

def inherit_from_data(name: str, delegate: Any, cache: bool=..., wrap: bool=...) -> Any: ...
def inherit_names(names: List[str], delegate: Any, cache: bool=..., wrap: bool=...) -> Any: ...
def make_wrapped_arith_op(opname: Any) -> Any: ...

class ExtensionIndex(Index):
    __eq__: Any = ...
    __ne__: Any = ...
    __lt__: Any = ...
    __gt__: Any = ...
    __le__: Any = ...
    __ge__: Any = ...
    def __getitem__(self, key: Any) -> Any: ...
    def __iter__(self) -> Any: ...
    def dropna(self, how: str = ...) -> Any: ...
    def repeat(self, repeats: Any, axis: Optional[Any] = ...) -> Any: ...
    def take(self, indices: Any, axis: int = ..., allow_fill: bool = ..., fill_value: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def unique(self, level: Optional[Any] = ...) -> Any: ...
    def map(self, mapper: Any, na_action: Optional[Any] = ...) -> Any: ...
    def astype(self, dtype: Any, copy: bool = ...) -> Any: ...



from typing import Any, Callable, Dict, Optional

def get_keywords() -> Any: ...

class VersioneerConfig: ...

def get_config() -> Any: ...

class NotThisMethod(Exception): ...

HANDLERS: Dict[str, Dict[str, Callable[..., Any]]]

def register_vcs_handler(vcs: str, method: str) -> Callable[..., Any]: ...
def run_command(commands: Any, args: Any, cwd: Optional[Any] = ..., verbose: bool = ..., hide_stderr: bool = ...) -> Any: ...
def versions_from_parentdir(parentdir_prefix: Any, root: Any, verbose: Any) -> Any: ...
def git_get_keywords(versionfile_abs: Any) -> Any: ...
def git_versions_from_keywords(keywords: Any, tag_prefix: Any, verbose: Any) -> Any: ...
def git_pieces_from_vcs(tag_prefix: Any, root: Any, verbose: Any, run_command: Any = ...) -> Any: ...
def plus_or_dot(pieces: Any) -> Any: ...
def render_pep440(pieces: Any) -> Any: ...
def render_pep440_pre(pieces: Any) -> Any: ...
def render_pep440_post(pieces: Any) -> Any: ...
def render_pep440_old(pieces: Any) -> Any: ...
def render_git_describe(pieces: Any) -> Any: ...
def render_git_describe_long(pieces: Any) -> Any: ...
def render(pieces: Any, style: Any) -> Any: ...
def get_versions() -> Any: ...



from pandas._config import get_option as get_option
from pandas.util._decorators import Appender as Appender
from typing import Any, Optional, Callable, TypeVar, Tuple, Union, overload

jinja2: Any
has_mpl: bool
no_mpl_message: str
PipeReturn = TypeVar('PipeReturn')

class Styler:
    loader: Any = ...
    env: Any = ...
    template: Any = ...
    ctx: Any = ...
    data: Any = ...
    index: Any = ...
    columns: Any = ...
    uuid: Any = ...
    table_styles: Any = ...
    caption: Any = ...
    precision: Any = ...
    table_attributes: Any = ...
    hidden_index: bool = ...
    hidden_columns: Any = ...
    cell_ids: Any = ...
    na_rep: Any = ...
    def __init__(self, data: Any, precision: Any = ..., table_styles: Any = ..., uuid: Any = ..., caption: Any = ..., table_attributes: Any = ..., cell_ids: Any = ..., na_rep: Optional[str]=...) -> None: ...
    def to_excel(self, excel_writer: Any, sheet_name: str = ..., na_rep: str = ..., float_format: Optional[Any] = ..., columns: Optional[Any] = ..., header: bool = ..., index: bool = ..., index_label: Optional[Any] = ..., startrow: int = ..., startcol: int = ..., engine: Optional[Any] = ..., merge_cells: bool = ..., encoding: Optional[Any] = ..., inf_rep: str = ..., verbose: bool = ..., freeze_panes: Optional[Any] = ...) -> None: ...
    def format(self, formatter: Any, subset: Any = ..., na_rep: Optional[str]=...) -> Any: ...
    def render(self, **kwargs: Any) -> Any: ...
    def __copy__(self) -> Any: ...
    def __deepcopy__(self, memo: Any) -> Any: ...
    def clear(self) -> None: ...
    def apply(self, func: Any, axis: int = ..., subset: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def applymap(self, func: Any, subset: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def where(self, cond: Any, value: Any, other: Optional[Any] = ..., subset: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def set_precision(self, precision: Any) -> Any: ...
    def set_table_attributes(self, attributes: Any) -> Any: ...
    def export(self) -> Any: ...
    def use(self, styles: Any) -> Any: ...
    def set_uuid(self, uuid: Any) -> Any: ...
    def set_caption(self, caption: Any) -> Any: ...
    def set_table_styles(self, table_styles: Any) -> Any: ...
    def set_na_rep(self, na_rep: str) -> Styler: ...
    def hide_index(self) -> Any: ...
    def hide_columns(self, subset: Any) -> Any: ...
    def highlight_null(self, null_color: str = ...) -> Any: ...
    def background_gradient(self, cmap: Any = ..., low: Any = ..., high: Any = ..., axis: Any = ..., subset: Any = ..., text_color_threshold: Any = ..., vmin: Optional[float]=..., vmax: Optional[float]=...) -> Any: ...
    def set_properties(self, subset: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def bar(self, subset: Optional[Any] = ..., axis: int = ..., color: str = ..., width: int = ..., align: str = ..., vmin: Optional[Any] = ..., vmax: Optional[Any] = ...) -> Any: ...
    def highlight_max(self, subset: Optional[Any] = ..., color: str = ..., axis: int = ...) -> Any: ...
    def highlight_min(self, subset: Optional[Any] = ..., color: str = ..., axis: int = ...) -> Any: ...
    @classmethod
    def from_custom_template(cls, searchpath: Any, name: Any) -> Any: ...
    @overload
    def pipe(self: Any, func: Union[Callable[..., PipeReturn], Tuple[Callable[..., PipeReturn], str]], *args: Any, **kwargs: Any) -> PipeReturn: ...
    @overload
    def pipe(self: Any, func: PipeReturn, *args: Any, **kwargs: Any) -> PipeReturn: ...



from pandas._libs import tslibs as tslibs
from pandas.core.dtypes.cast import maybe_promote as maybe_promote

from pandas.core.dtypes.missing import isna as isna
from pandas.util._decorators import cache_readonly as cache_readonly
from typing import Any, Optional

def get_mgr_concatenation_plan(mgr: Any, indexers: Any) -> Any: ...

class JoinUnit:
    block: Any = ...
    indexers: Any = ...
    shape: Any = ...
    def __init__(self, block: Any, shape: Any, indexers: Optional[Any] = ...) -> None: ...
    def needs_filling(self) -> Any: ...
    def dtype(self) -> Any: ...
    def is_na(self) -> Any: ...
    def get_reindexed_values(self, empty_dtype: Any, upcasted_na: Any) -> Any: ...

def concatenate_join_units(join_units: Any, concat_axis: Any, copy: Any) -> Any: ...
def is_uniform_join_units(join_units: Any) -> Any: ...
def combine_concat_plans(plans: Any, concat_axis: Any) -> Any: ...



from pandas._libs.tslibs import parsing as parsing
from typing import Any

def parse_date_time(date_col: Any, time_col: Any) -> Any: ...
def parse_date_fields(year_col: Any, month_col: Any, day_col: Any) -> Any: ...
def parse_all_fields(year_col: Any, month_col: Any, day_col: Any, hour_col: Any, minute_col: Any, second_col: Any) -> Any: ...
def generic_parser(parse_func: Any, *cols: Any) -> Any: ...



from pandas.util._decorators import Appender as Appender
from typing import Any

class DirNamesMixin:
    def __dir__(self) -> Any: ...

class PandasDelegate: ...

def delegate_names(delegate: Any, accessors: Any, typ: str, overwrite: bool=...) -> Any: ...

class CachedAccessor:
    def __init__(self, name: str, accessor: Any) -> None: ...
    def __get__(self, obj: Any, cls: Any) -> Any: ...

def register_dataframe_accessor(name: Any) -> Any: ...
def register_series_accessor(name: Any) -> Any: ...
def register_index_accessor(name: Any) -> Any: ...



import numpy as np
from numpy.lib.mixins import NDArrayOperatorsMixin
from pandas import compat as compat
from pandas.core import nanops as nanops
from pandas.core.algorithms import searchsorted as searchsorted, take as take, unique as unique
from pandas.core.arrays.base import ExtensionArray as ExtensionArray, ExtensionOpsMixin as ExtensionOpsMixin
from pandas.core.construction import extract_array as extract_array
from pandas.core.dtypes.dtypes import ExtensionDtype as ExtensionDtype
from pandas.core.dtypes.generic import ABCIndexClass as ABCIndexClass, ABCSeries as ABCSeries
from pandas.core.dtypes.inference import is_array_like as is_array_like
from pandas.core.dtypes.missing import isna as isna

from pandas.core.missing import backfill_1d as backfill_1d, pad_1d as pad_1d
from pandas.util._decorators import Appender as Appender
from pandas.util._validators import validate_fillna_kwargs as validate_fillna_kwargs
from typing import Any, Optional, Union

class PandasDtype(ExtensionDtype):
    def __init__(self, dtype: Any) -> None: ...
    @property
    def numpy_dtype(self) -> Any: ...
    @property
    def name(self) -> Any: ...
    @property
    def type(self) -> Any: ...
    @classmethod
    def construct_from_string(cls, string: Any) -> Any: ...
    @classmethod
    def construct_array_type(cls) -> Any: ...
    @property
    def kind(self) -> Any: ...
    @property
    def itemsize(self) -> Any: ...

class PandasArray(ExtensionArray, ExtensionOpsMixin, NDArrayOperatorsMixin):    # type: ignore
    __array_priority__: int = ...
    def __init__(self, values: Union[np.ndarray, PandasArray], copy: bool=...) -> None: ...
    @property
    def dtype(self) -> Any: ...
    def __array__(self, dtype: Any = ...) -> np.ndarray: ...
    def __array_ufunc__(self, ufunc: Any, method: Any, *inputs: Any, **kwargs: Any) -> Any: ...
    def __getitem__(self, item: Any) -> Any: ...
    def __setitem__(self, key: Any, value: Any) -> None: ...
    def __len__(self) -> int: ...
    @property
    def nbytes(self) -> int: ...
    def isna(self) -> Any: ...
    def fillna(self, value: Optional[Any] = ..., method: Optional[Any] = ..., limit: Optional[Any] = ...) -> Any: ...
    def take(self, indices: Any, allow_fill: bool = ..., fill_value: Optional[Any] = ...) -> Any: ...
    def copy(self) -> Any: ...
    def unique(self) -> Any: ...
    def any(self, axis: Optional[Any] = ..., out: Optional[Any] = ..., keepdims: bool = ..., skipna: bool = ...) -> Any: ...
    def all(self, axis: Optional[Any] = ..., out: Optional[Any] = ..., keepdims: bool = ..., skipna: bool = ...) -> Any: ...
    def min(self, axis: Optional[Any] = ..., out: Optional[Any] = ..., keepdims: bool = ..., skipna: bool = ...) -> Any: ...
    def max(self, axis: Optional[Any] = ..., out: Optional[Any] = ..., keepdims: bool = ..., skipna: bool = ...) -> Any: ...
    def sum(self, axis: Optional[Any] = ..., dtype: Optional[Any] = ..., out: Optional[Any] = ..., keepdims: bool = ..., initial: Optional[Any] = ..., skipna: bool = ..., min_count: int = ...) -> Any: ...
    def prod(self, axis: Optional[Any] = ..., dtype: Optional[Any] = ..., out: Optional[Any] = ..., keepdims: bool = ..., initial: Optional[Any] = ..., skipna: bool = ..., min_count: int = ...) -> Any: ...
    def mean(self, axis: Optional[Any] = ..., dtype: Optional[Any] = ..., out: Optional[Any] = ..., keepdims: bool = ..., skipna: bool = ...) -> Any: ...
    def median(self, axis: Optional[Any] = ..., out: Optional[Any] = ..., overwrite_input: bool = ..., keepdims: bool = ..., skipna: bool = ...) -> Any: ...
    def std(self, axis: Optional[Any] = ..., dtype: Optional[Any] = ..., out: Optional[Any] = ..., ddof: int = ..., keepdims: bool = ..., skipna: bool = ...) -> Any: ...
    def var(self, axis: Optional[Any] = ..., dtype: Optional[Any] = ..., out: Optional[Any] = ..., ddof: int = ..., keepdims: bool = ..., skipna: bool = ...) -> Any: ...
    def sem(self, axis: Optional[Any] = ..., dtype: Optional[Any] = ..., out: Optional[Any] = ..., ddof: int = ..., keepdims: bool = ..., skipna: bool = ...) -> Any: ...
    def kurt(self, axis: Optional[Any] = ..., dtype: Optional[Any] = ..., out: Optional[Any] = ..., keepdims: bool = ..., skipna: bool = ...) -> Any: ...
    def skew(self, axis: Optional[Any] = ..., dtype: Optional[Any] = ..., out: Optional[Any] = ..., keepdims: bool = ..., skipna: bool = ...) -> Any: ...
    def to_numpy(self, dtype: Optional[Any] = ..., copy: bool = ..., na_value: Any = ...) -> Any: ...
    def searchsorted(self, value: Any, side: str = ..., sorter: Optional[Any] = ...) -> Any: ...
    def __invert__(self) -> Any: ...



from typing import Any, Optional

def get_dtype_kinds(l: Any) -> Any: ...
def concat_compat(to_concat: Any, axis: int=...) -> Any: ...
def concat_categorical(to_concat: Any, axis: int=...) -> Any: ...
def union_categoricals(to_union: Any, sort_categories: bool=..., ignore_order: bool=...) -> Any: ...
def concat_datetime(to_concat: Any, axis: int = ..., typs: Optional[Any] = ...) -> Any: ...



import numpy.ma as np
from pandas.core import algorithms as algorithms
from pandas.core.arrays import Categorical as Categorical
from pandas.core.construction import sanitize_array as sanitize_array
from pandas.core.dtypes.cast import construct_1d_arraylike_from_scalar as construct_1d_arraylike_from_scalar, maybe_cast_to_datetime as maybe_cast_to_datetime, maybe_convert_platform as maybe_convert_platform, maybe_infer_to_datetimelike as maybe_infer_to_datetimelike, maybe_upcast as maybe_upcast

from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame, ABCDatetimeIndex as ABCDatetimeIndex, ABCIndexClass as ABCIndexClass, ABCPeriodIndex as ABCPeriodIndex, ABCSeries as ABCSeries, ABCTimedeltaIndex as ABCTimedeltaIndex
from pandas.core.indexes.api import Index as Index, ensure_index as ensure_index, get_objs_combined_axis as get_objs_combined_axis, union_indexes as union_indexes
from pandas.core.internals import create_block_manager_from_arrays as create_block_manager_from_arrays, create_block_manager_from_blocks as create_block_manager_from_blocks
from typing import Any, Optional

def arrays_to_mgr(arrays: Any, arr_names: Any, index: Any, columns: Any, dtype: Optional[Any] = ...) -> Any: ...
def masked_rec_array_to_mgr(data: Any, index: Any, columns: Any, dtype: Any, copy: Any) -> Any: ...
def init_ndarray(values: Any, index: Any, columns: Any, dtype: Optional[Any] = ..., copy: bool = ...) -> Any: ...
def init_dict(data: Any, index: Any, columns: Any, dtype: Optional[Any] = ...) -> Any: ...
def prep_ndarray(values: Any, copy: Any = ...) -> np.ndarray: ...
def extract_index(data: Any) -> Any: ...
def reorder_arrays(arrays: Any, arr_columns: Any, columns: Any) -> Any: ...
def get_names_from_index(data: Any) -> Any: ...
def to_arrays(data: Any, columns: Any, coerce_float: bool = ..., dtype: Optional[Any] = ...) -> Any: ...
def sanitize_index(data: Any, index: Any, copy: bool = ...) -> Any: ...



from typing import Any, Callable, Optional

def safe_import(mod_name: str, min_version: Optional[str]=...) -> Any: ...

tables: Any
xfail_non_writeable: Any

def skip_if_installed(package: str) -> Callable[..., Any]: ...
def skip_if_no(package: str, min_version: Optional[str]=...) -> Callable[..., Any]: ...

skip_if_no_mpl: Any
skip_if_mpl: Any
skip_if_32bit: Any
skip_if_windows: Any
skip_if_windows_python_3: Any
skip_if_has_locale: Any
skip_if_not_us_locale: Any
skip_if_no_scipy: Any
skip_if_no_ne: Any

def skip_if_np_lt(ver_str: str, reason: Optional[str]=..., *args: Any, **kwds: Any) -> Callable[..., Any]: ...
def parametrize_fixture_doc(*args: Any) -> Any: ...
def check_file_leaks(func: Any) -> Callable[..., Any]: ...
def async_mark() -> Any: ...



from collections import namedtuple
from typing import Any, Optional

DeprecatedOption = namedtuple('DeprecatedOption', 'key msg rkey removal_ver')

RegisteredOption = namedtuple('RegisteredOption', 'key defval doc validator cb')

class OptionError(AttributeError, KeyError): ...

def get_default_val(pat: Any) -> Any: ...

class DictWrapper:
    def __init__(self, d: Any, prefix: str = ...) -> None: ...
    def __setattr__(self, key: Any, val: Any) -> None: ...
    def __getattr__(self, key: str) -> Any: ...
    def __dir__(self) -> Any: ...

class CallableDynamicDoc:
    __doc_tmpl__: Any = ...
    __func__: Any = ...
    def __init__(self, func: Any, doc_tmpl: Any) -> None: ...
    def __call__(self, *args: Any, **kwds: Any) -> Any: ...

get_option: Any
set_option: Any
reset_option: Any
describe_option: Any
options: Any

class option_context:
    ops: Any = ...
    def __init__(self, *args: Any) -> None: ...
    undo: Any = ...
    def __enter__(self) -> None: ...
    def __exit__(self, *args: Any) -> None: ...

def register_option(key: str, defval: object, doc: Any = ..., validator: Any = ..., cb: Any = ...) -> Any: ...
def deprecate_option(key: Any, msg: Optional[Any] = ..., rkey: Optional[Any] = ..., removal_ver: Optional[Any] = ...) -> None: ...
def pp_options_list(keys: Any, width: int = ..., _print: bool = ...) -> Any: ...
def config_prefix(prefix: Any) -> Any: ...
def is_type_factory(_type: Any) -> Any: ...
def is_instance_factory(_type: Any) -> Any: ...
def is_one_of_factory(legal_values: Any) -> Any: ...
def is_nonnegative_int(value: Any) -> None: ...

is_int: Any
is_bool: Any
is_float: Any
is_str: Any
is_text: Any

def is_callable(obj: Any) -> Any: ...




from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame, ABCIndexClass as ABCIndexClass, ABCSeries as ABCSeries
from pandas.plotting._matplotlib import compat as compat
from typing import Any, Optional

def format_date_labels(ax: Any, rot: Any) -> None: ...
def table(ax: Any, data: Any, rowLabels: Optional[Any] = ..., colLabels: Optional[Any] = ..., **kwargs: Any) -> Any: ...



import numpy as np
from pandas.core.arrays import ExtensionArray
from pandas.core.base import IndexOpsMixin, PandasObject
from typing import Any, Dict, Hashable, Optional, Sequence


class InvalidIndexError(Exception): ...

class Index(IndexOpsMixin, PandasObject):
    str: Any = ...
    def __new__(cls: Any, data: Any = ..., dtype: Any = ..., copy: Any = ..., name: Any = ..., tupleize_cols: Any = ..., **kwargs: Any) -> Index: ...
    @property
    def asi8(self) -> None: ...
    def is_(self, other: Any) -> bool: ...
    def __len__(self) -> int: ...
    def __array__(self, dtype: Any = ...) -> np.ndarray: ...
    def __array_wrap__(self, result: Any, context: Optional[Any] = ...) -> Any: ...
    def dtype(self) -> Any: ...
    def ravel(self, order: str = ...) -> Any: ...
    def view(self, cls: Optional[Any] = ...) -> Any: ...
    def astype(self, dtype: Any, copy: bool = ...) -> Any: ...
    def take(self, indices: Any, axis: int = ..., allow_fill: bool = ..., fill_value: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def repeat(self, repeats: Any, axis: Optional[Any] = ...) -> Any: ...
    def copy(self, name: Optional[Any] = ..., deep: bool = ..., dtype: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def __copy__(self, **kwargs: Any) -> Any: ...
    def __deepcopy__(self, memo: Optional[Any] = ...) -> Any: ...
    def format(self, name: bool = ..., formatter: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def to_native_types(self, slicer: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def to_flat_index(self) -> Any: ...
    def to_series(self, index: Optional[Any] = ..., name: Optional[Any] = ...) -> Any: ...
    def to_frame(self, index: bool = ..., name: Optional[Any] = ...) -> Any: ...
    @property
    def name(self) -> Any: ...
    @name.setter
    def name(self, value: Any) -> None: ...
    names: Any = ...
    def set_names(self, names: Any, level: Optional[Any] = ..., inplace: bool = ...) -> Any: ...
    def rename(self, name: Any, inplace: bool = ...) -> Any: ...
    @property
    def nlevels(self) -> int: ...
    def sortlevel(self, level: Optional[Any] = ..., ascending: bool = ..., sort_remaining: Optional[Any] = ...) -> Any: ...
    get_level_values: Any = ...
    def droplevel(self, level: int = ...) -> Any: ...
    @property
    def is_monotonic(self) -> bool: ...
    @property
    def is_monotonic_increasing(self) -> Any: ...
    @property
    def is_monotonic_decreasing(self) -> bool: ...
    def is_unique(self) -> bool: ...
    @property
    def has_duplicates(self) -> bool: ...
    def is_boolean(self) -> bool: ...
    def is_integer(self) -> bool: ...
    def is_floating(self) -> bool: ...
    def is_numeric(self) -> bool: ...
    def is_object(self) -> bool: ...
    def is_categorical(self) -> bool: ...
    def is_interval(self) -> bool: ...
    def is_mixed(self) -> bool: ...
    def holds_integer(self) -> Any: ...
    def inferred_type(self) -> Any: ...
    def is_all_dates(self) -> bool: ...
    def __reduce__(self) -> Any: ...
    def hasnans(self) -> Any: ...
    def isna(self) -> Any: ...
    isnull: Any = ...
    def notna(self) -> Any: ...
    notnull: Any = ...
    def fillna(self, value: Optional[Any] = ..., downcast: Optional[Any] = ...) -> Any: ...
    def dropna(self, how: str = ...) -> Any: ...
    def unique(self, level: Optional[Any] = ...) -> Any: ...
    def drop_duplicates(self, keep: str = ...) -> Any: ...  # type: ignore
    def duplicated(self, keep: str = ...) -> Any: ...
    def __add__(self, other: Any) -> Any: ...
    def __radd__(self, other: Any) -> Any: ...
    def __iadd__(self, other: Any) -> Any: ...
    def __sub__(self, other: Any) -> Any: ...
    def __rsub__(self, other: Any) -> Any: ...
    def __and__(self, other: Any) -> Any: ...
    def __or__(self, other: Any) -> Any: ...
    def __xor__(self, other: Any) -> Any: ...
    def __nonzero__(self) -> None: ...
    __bool__: Any = ...
    def union(self, other: Any, sort: Optional[Any] = ...) -> Any: ...
    def intersection(self, other: Any, sort: bool = ...) -> Any: ...
    def difference(self, other: Any, sort: Optional[Any] = ...) -> Any: ...
    def symmetric_difference(self, other: Any, result_name: Optional[Any] = ..., sort: Optional[Any] = ...) -> Any: ...
    def get_loc(self, key: Any, method: Optional[Any] = ..., tolerance: Optional[Any] = ...) -> Any: ...
    def get_indexer(self, target: Any, method: Optional[Any] = ..., limit: Optional[Any] = ..., tolerance: Optional[Any] = ...) -> Any: ...
    def reindex(self, target: Any, method: Optional[Any] = ..., level: Optional[Any] = ..., limit: Optional[Any] = ..., tolerance: Optional[Any] = ...) -> Any: ...
    def join(self, other: Any, how: str = ..., level: Optional[Any] = ..., return_indexers: bool = ..., sort: bool = ...) -> Any: ...
    @property
    def values(self) -> Any: ...
    def array(self) -> ExtensionArray: ...
    def memory_usage(self, deep: bool = ...) -> int: ...
    def where(self, cond: Any, other: Optional[Any] = ...) -> Any: ...
    def is_type_compatible(self, kind: Any) -> bool: ...
    def __contains__(self, key: Any) -> bool: ...
    def __hash__(self) -> Any: ...
    def __setitem__(self, key: Any, value: Any) -> None: ...
    def __getitem__(self, key: Any) -> Any: ...
    def append(self, other: Any) -> Any: ...
    def putmask(self, mask: Any, value: Any) -> Any: ...
    def equals(self, other: Any) -> bool: ...
    def identical(self, other: Any) -> bool: ...
    def asof(self, label: Any) -> Any: ...
    def asof_locs(self, where: Any, mask: Any) -> Any: ...
    def sort_values(self, return_indexer: bool = ..., ascending: bool = ...) -> Any: ...
    def sort(self, *args: Any, **kwargs: Any) -> None: ...
    def shift(self, periods: int = ..., freq: Optional[Any] = ...) -> None: ...
    def argsort(self, *args: Any, **kwargs: Any) -> Any: ...
    def get_value(self, series: Any, key: Any) -> Any: ...
    def set_value(self, arr: Any, key: Any, value: Any) -> None: ...
    def get_indexer_non_unique(self, target: Any) -> Any: ...
    def get_indexer_for(self, target: Any, **kwargs: Any) -> Any: ...
    def groupby(self, values: Any) -> Dict[Hashable, np.ndarray]: ...
    def map(self, mapper: Any, na_action: Optional[Any] = ...) -> Any: ...
    def isin(self, values: Any, level: Optional[Any] = ...) -> Any: ...
    def slice_indexer(self, start: Optional[Any] = ..., end: Optional[Any] = ..., step: Optional[Any] = ..., kind: Optional[Any] = ...) -> Any: ...
    def get_slice_bound(self, label: Any, side: Any, kind: Any) -> Any: ...
    def slice_locs(self, start: Optional[Any] = ..., end: Optional[Any] = ..., step: Optional[Any] = ..., kind: Optional[Any] = ...) -> Any: ...
    def delete(self, loc: Any) -> Any: ...
    def insert(self, loc: Any, item: Any) -> Any: ...
    def drop(self, labels: Any, errors: str = ...) -> Any: ...
    @property
    def shape(self) -> Any: ...

def ensure_index(index_like: Any, copy: bool) -> None: ...

def maybe_extract_name(name: str, obj: Any, cls: Any) -> Optional[Hashable]: ...

def _new_Index(cls: Any, d: Any) -> Any: ...

def ensure_index_from_sequences(sequences: Any, names: Optional[Sequence[str]]) -> Any: ...


from pandas._libs import Timestamp as Timestamp
from pandas.core.dtypes.cast import infer_dtype_from_scalar as infer_dtype_from_scalar

from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame, ABCIndexClass as ABCIndexClass, ABCMultiIndex as ABCMultiIndex, ABCSeries as ABCSeries
from pandas.core.dtypes.missing import isna as isna
from typing import Any, Optional

def hash_pandas_object(obj: Any, index: bool=..., encoding: str=..., hash_key: Optional[str]=..., categorize: bool=...) -> Any: ...
def hash_tuples(vals: Any, encoding: Any = ..., hash_key: str=...) -> Any: ...
def hash_tuple(val: Any, encoding: str=..., hash_key: str=...) -> Any: ...
def hash_array(vals: Any, encoding: str=..., hash_key: str=..., categorize: bool=...) -> Any: ...



from pandas.io.json._json import dumps as dumps, loads as loads, read_json as read_json, to_json as to_json
from pandas.io.json._normalize import _json_normalize as _json_normalize, json_normalize as json_normalize
from pandas.io.json._table_schema import build_table_schema as build_table_schema



from pandas.core.algorithms import unique1d as unique1d
from pandas.core.arrays.categorical import Categorical as Categorical, CategoricalDtype as CategoricalDtype
from typing import Any

def recode_for_groupby(c: Categorical, sort: bool, observed: bool) -> Any: ...
def recode_from_groupby(c: Categorical, sort: bool, ci: Any) -> Any: ...



from collections import abc
from pandas import DataFrame as DataFrame, MultiIndex as MultiIndex, Series as Series, isna as isna, to_datetime as to_datetime
from pandas._typing import JSONSerializable as JSONSerializable
from pandas.core.construction import create_series_with_explicit_dtype as create_series_with_explicit_dtype
from pandas.core.dtypes.common import ensure_str as ensure_str, is_period_dtype as is_period_dtype
from pandas.core.reshape.concat import concat as concat
from pandas.errors import AbstractMethodError as AbstractMethodError
from pandas.io.common import get_filepath_or_buffer as get_filepath_or_buffer, get_handle as get_handle, infer_compression as infer_compression, stringify_path as stringify_path
from pandas.io.json._normalize import convert_to_line_delimits as convert_to_line_delimits
from pandas.io.json._table_schema import build_table_schema as build_table_schema, parse_table_schema as parse_table_schema
from pandas.util._decorators import deprecate_kwarg as deprecate_kwarg
from typing import Any, Callable, Optional

loads: Any
dumps: Any
TABLE_SCHEMA_VERSION: str

def to_json(path_or_buf: Any, obj: Any, orient: Optional[str]=..., date_format: str=..., double_precision: int=..., force_ascii: bool=..., date_unit: str=..., default_handler: Optional[Callable[[Any], JSONSerializable]]=..., lines: bool=..., compression: Optional[str]=..., index: bool=..., indent: int=...) -> Any: ...

class Writer:
    obj: Any = ...
    orient: Any = ...
    date_format: Any = ...
    double_precision: Any = ...
    ensure_ascii: Any = ...
    date_unit: Any = ...
    default_handler: Any = ...
    index: Any = ...
    indent: Any = ...
    is_copy: Any = ...
    def __init__(self, obj: Any, orient: Optional[str], date_format: str, double_precision: int, ensure_ascii: bool, date_unit: str, index: bool, default_handler: Optional[Callable[[Any], JSONSerializable]]=..., indent: int=...) -> None: ...
    def write(self) -> Any: ...

class SeriesWriter(Writer): ...
class FrameWriter(Writer): ...

class JSONTableWriter(FrameWriter):
    schema: Any = ...
    obj: Any = ...
    date_format: str = ...
    orient: str = ...
    index: Any = ...
    def __init__(self, obj: Any, orient: Optional[str], date_format: str, double_precision: int, ensure_ascii: bool, date_unit: str, index: bool, default_handler: Optional[Callable[[Any], JSONSerializable]]=..., indent: int=...) -> None: ...

def read_json(path_or_buf: Optional[Any] = ..., orient: Optional[Any] = ..., typ: str = ..., dtype: Optional[Any] = ..., convert_axes: Optional[Any] = ..., convert_dates: bool = ..., keep_default_dates: bool = ..., numpy: bool = ..., precise_float: bool = ..., date_unit: Optional[Any] = ..., encoding: Optional[Any] = ..., lines: bool = ..., chunksize: Optional[Any] = ..., compression: str = ...) -> Any: ...

class JsonReader(abc.Iterator[Any]):
    path_or_buf: Any = ...
    orient: Any = ...
    typ: Any = ...
    dtype: Any = ...
    convert_axes: Any = ...
    convert_dates: Any = ...
    keep_default_dates: Any = ...
    numpy: Any = ...
    precise_float: Any = ...
    date_unit: Any = ...
    encoding: Any = ...
    compression: Any = ...
    lines: Any = ...
    chunksize: Any = ...
    nrows_seen: int = ...
    should_close: bool = ...
    data: Any = ...
    def __init__(self, filepath_or_buffer: Any, orient: Any, typ: Any, dtype: Any, convert_axes: Any, convert_dates: Any, keep_default_dates: Any, numpy: Any, precise_float: Any, date_unit: Any, encoding: Any, lines: Any, chunksize: Any, compression: Any) -> None: ...
    def read(self) -> Any: ...
    def close(self) -> None: ...
    def __next__(self) -> Any: ...

class Parser:
    json: Any = ...
    orient: Any = ...
    dtype: Any = ...
    min_stamp: Any = ...
    numpy: Any = ...
    precise_float: Any = ...
    convert_axes: Any = ...
    convert_dates: Any = ...
    date_unit: Any = ...
    keep_default_dates: Any = ...
    obj: Any = ...
    def __init__(self, json: Any, orient: Any, dtype: Optional[Any] = ..., convert_axes: bool = ..., convert_dates: bool = ..., keep_default_dates: bool = ..., numpy: bool = ..., precise_float: bool = ..., date_unit: Optional[Any] = ...) -> None: ...
    def check_keys_split(self, decoded: Any) -> None: ...
    def parse(self) -> Any: ...

class SeriesParser(Parser): ...
class FrameParser(Parser): ...



from pandas._config import get_option as get_option
from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame
from typing import Any, Optional

def set_use_numexpr(v: bool = ...) -> None: ...
def set_numexpr_threads(n: Optional[Any] = ...) -> None: ...
def evaluate(op: Any, op_str: Any, a: Any, b: Any, use_numexpr: bool = ...) -> Any: ...
def where(cond: Any, a: Any, b: Any, use_numexpr: bool = ...) -> Any: ...
def set_test_mode(v: bool = ...) -> None: ...
def get_test_result() -> Any: ...



import numpy as np
from pandas._typing import FrameOrSeries as FrameOrSeries
from pandas.core.arrays import Categorical as Categorical, ExtensionArray as ExtensionArray
from pandas.core.dtypes.common import ensure_categorical as ensure_categorical, is_categorical_dtype as is_categorical_dtype, is_datetime64_dtype as is_datetime64_dtype, is_list_like as is_list_like, is_scalar as is_scalar, is_timedelta64_dtype as is_timedelta64_dtype
from pandas.core.dtypes.generic import ABCSeries as ABCSeries
from pandas.core.frame import DataFrame as DataFrame
from pandas.core.groupby import ops as ops
from pandas.core.groupby.categorical import recode_for_groupby as recode_for_groupby, recode_from_groupby as recode_from_groupby
from pandas.core.indexes.api import CategoricalIndex as CategoricalIndex, Index as Index, MultiIndex as MultiIndex
from pandas.core.series import Series as Series
from pandas.io.formats.printing import pprint_thing as pprint_thing
from pandas.util._decorators import cache_readonly as cache_readonly
from typing import Any, Dict, Hashable, List, Optional, Tuple

class Grouper:
    def __new__(cls, *args: Any, **kwargs: Any) -> Any: ...
    key: Any = ...
    level: Any = ...
    freq: Any = ...
    axis: Any = ...
    sort: Any = ...
    grouper: Any = ...
    obj: Any = ...
    indexer: Any = ...
    binner: Any = ...
    def __init__(self, key: Optional[Any] = ..., level: Optional[Any] = ..., freq: Optional[Any] = ..., axis: int = ..., sort: bool = ...) -> None: ...
    @property
    def ax(self) -> Any: ...
    @property
    def groups(self) -> Any: ...

class Grouping:
    name: Any = ...
    level: Any = ...
    grouper: Any = ...
    all_grouper: Any = ...
    index: Any = ...
    sort: Any = ...
    obj: Any = ...
    observed: Any = ...
    in_axis: Any = ...
    def __init__(self, index: Index, grouper: Any = ..., obj: Optional[FrameOrSeries]=..., name: Any = ..., level: Any = ..., sort: bool=..., observed: bool=..., in_axis: bool=...) -> None: ...
    def __iter__(self) -> Any: ...
    @property
    def ngroups(self) -> int: ...
    def indices(self) -> Any: ...
    @property
    def codes(self) -> np.ndarray: ...
    def result_index(self) -> Index: ...
    @property
    def group_index(self) -> Index: ...
    def groups(self) -> Dict[Hashable, np.ndarray]: ...

def get_grouper(obj: FrameOrSeries, key: Any = ..., axis: int=..., level: Any = ..., sort: bool=..., observed: bool=..., mutated: bool=..., validate: bool=...) -> Tuple[ops.BaseGrouper, List[Hashable], FrameOrSeries]: ...



import numpy as np
from pandas.core.construction import array as array

from pandas.core.dtypes.generic import ABCExtensionArray as ABCExtensionArray, ABCSeries as ABCSeries
from typing import Any, Union

def should_extension_dispatch(left: ABCSeries, right: Any) -> bool: ...
def should_series_dispatch(left: Any, right: Any, op: Any) -> Any: ...
def dispatch_to_extension_op(op: Any, left: Union[ABCExtensionArray, np.ndarray], right: Any) -> Any: ...



from typing import Any

class PerformanceWarning(Warning): ...
class UnsupportedFunctionCall(ValueError): ...
class UnsortedIndexError(KeyError): ...
class ParserError(ValueError): ...
class DtypeWarning(Warning): ...
class EmptyDataError(ValueError): ...
class ParserWarning(Warning): ...
class MergeError(ValueError): ...
class AccessorRegistrationWarning(Warning): ...

class AbstractMethodError(NotImplementedError):
    methodtype: Any = ...
    class_instance: Any = ...
    def __init__(self, class_instance: Any, methodtype: str = ...) -> None: ...



from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame, ABCIndexClass as ABCIndexClass
from pandas.plotting._matplotlib.core import LinePlot as LinePlot, MPLPlot as MPLPlot
from typing import Any, Optional

class HistPlot(LinePlot):
    bins: Any = ...
    bottom: Any = ...
    def __init__(self, data: Any, bins: int = ..., bottom: int = ..., **kwargs: Any) -> None: ...
    @property
    def orientation(self) -> Any: ...  # type: ignore

class KdePlot(HistPlot):
    orientation: str = ...
    bw_method: Any = ...
    ind: Any = ...
    def __init__(self, data: Any, bw_method: Optional[Any] = ..., ind: Optional[Any] = ..., **kwargs: Any) -> None: ...

def hist_series(self: Any, by: Optional[Any] = ..., ax: Optional[Any] = ..., grid: bool = ..., xlabelsize: Optional[Any] = ..., xrot: Optional[Any] = ..., ylabelsize: Optional[Any] = ..., yrot: Optional[Any] = ..., figsize: Optional[Any] = ..., bins: int = ..., **kwds: Any) -> Any: ...
def hist_frame(data: Any, column: Optional[Any] = ..., by: Optional[Any] = ..., grid: bool = ..., xlabelsize: Optional[Any] = ..., xrot: Optional[Any] = ..., ylabelsize: Optional[Any] = ..., yrot: Optional[Any] = ..., ax: Optional[Any] = ..., sharex: bool = ..., sharey: bool = ..., figsize: Optional[Any] = ..., layout: Optional[Any] = ..., bins: int = ..., **kwds: Any) -> Any: ...



from pandas.core.indexes.base import Index as Index, InvalidIndexError as InvalidIndexError, _new_Index as _new_Index, ensure_index as ensure_index, ensure_index_from_sequences as ensure_index_from_sequences
from pandas.core.indexes.category import CategoricalIndex as CategoricalIndex
from pandas.core.indexes.datetimes import DatetimeIndex as DatetimeIndex
from pandas.core.indexes.interval import IntervalIndex as IntervalIndex
from pandas.core.indexes.multi import MultiIndex as MultiIndex
from pandas.core.indexes.numeric import Float64Index as Float64Index, Int64Index as Int64Index, NumericIndex as NumericIndex, UInt64Index as UInt64Index
from pandas.core.indexes.period import PeriodIndex as PeriodIndex
from pandas.core.indexes.range import RangeIndex as RangeIndex
from pandas.core.indexes.timedeltas import TimedeltaIndex as TimedeltaIndex
from typing import Any

def get_objs_combined_axis(objs: Any, intersect: bool=..., axis: Any = ..., sort: bool=...) -> Index: ...
def union_indexes(indexes: Any, sort: Any = ...) -> Index: ...
def get_consensus_names(indexes: Any) -> Any: ...
def all_indexes_same(indexes: Any) -> Any: ...



import numpy.ma as np
from pandas._typing import ArrayLike as ArrayLike, Dtype as Dtype
from pandas.core.dtypes.cast import construct_1d_arraylike_from_scalar as construct_1d_arraylike_from_scalar, construct_1d_ndarray_preserving_na as construct_1d_ndarray_preserving_na, construct_1d_object_array_from_listlike as construct_1d_object_array_from_listlike, infer_dtype_from_scalar as infer_dtype_from_scalar, maybe_cast_to_datetime as maybe_cast_to_datetime, maybe_cast_to_integer_array as maybe_cast_to_integer_array, maybe_castable as maybe_castable, maybe_convert_platform as maybe_convert_platform, maybe_upcast as maybe_upcast

from pandas.core.dtypes.dtypes import CategoricalDtype as CategoricalDtype, ExtensionDtype as ExtensionDtype, registry as registry
from pandas.core.dtypes.generic import ABCExtensionArray as ABCExtensionArray, ABCIndexClass as ABCIndexClass, ABCPandasArray as ABCPandasArray, ABCSeries as ABCSeries
from pandas.core.dtypes.missing import isna as isna
from pandas.core.indexes.api import Index as Index
from pandas.core.series import Series as Series
from typing import Any, Optional, Sequence, Union

def array(data: Sequence[object], dtype: Optional[Union[str, np.dtype, ExtensionDtype]]=..., copy: bool=...) -> ABCExtensionArray: ...
def extract_array(obj: Any, extract_numpy: bool = ...) -> Any: ...
def sanitize_array(data: Any, index: Any, dtype: Any = ..., copy: bool=..., raise_cast_failure: bool=...) -> Any: ...
def is_empty_data(data: Any) -> bool: ...
def create_series_with_explicit_dtype(data: Any = ..., index: Optional[Union[ArrayLike, Index]]=..., dtype: Optional[Dtype]=..., name: Optional[str]=..., copy: bool=..., fastpath: bool=..., dtype_if_empty: Dtype=...) -> Series: ...



from pandas import DataFrame as DataFrame
from pandas.core.dtypes.cast import maybe_downcast_to_dtype as maybe_downcast_to_dtype

from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame, ABCSeries as ABCSeries
from pandas.core.groupby import Grouper as Grouper
from pandas.core.indexes.api import Index as Index, MultiIndex as MultiIndex, get_objs_combined_axis as get_objs_combined_axis
from pandas.core.reshape.concat import concat as concat
from pandas.core.reshape.util import cartesian_product as cartesian_product
from pandas.core.series import Series as Series
from pandas.util._decorators import Appender as Appender, Substitution as Substitution
from typing import Any

def pivot_table(data: Any, values: Any = ..., index: Any = ..., columns: Any = ..., aggfunc: Any = ..., fill_value: Any = ..., margins: Any = ..., dropna: Any = ..., margins_name: Any = ..., observed: Any = ...) -> DataFrame: ...
def pivot(data: DataFrame, index: Any = ..., columns: Any = ..., values: Any = ...) -> DataFrame: ...
def crosstab(index: Any, columns: Any, values: Any = ..., rownames: Any = ..., colnames: Any = ..., aggfunc: Any = ..., margins: Any = ..., margins_name: str=..., dropna: bool=..., normalize: Any = ...) -> DataFrame: ...



import zipfile
from collections import abc
from io import BytesIO
from pandas._typing import FilePathOrBuffer as FilePathOrBuffer

from pandas.errors import AbstractMethodError as AbstractMethodError, DtypeWarning as DtypeWarning, EmptyDataError as EmptyDataError, ParserError as ParserError, ParserWarning as ParserWarning
from typing import Any, AnyStr, Dict, IO, Mapping, Optional, Tuple, Union, TypeVar
from urllib.parse import urlencode as urlencode

lzma: Any

T = TypeVar('T')

def is_url(url: Any) -> bool: ...
def validate_header_arg(header: Any) -> None: ...
def stringify_path(filepath_or_buffer: FilePathOrBuffer[AnyStr]) -> FilePathOrBuffer[AnyStr]: ...
def is_s3_url(url: Any) -> bool: ...
def is_gcs_url(url: Any) -> bool: ...
def urlopen(*args: Any, **kwargs: Any) -> Any: ...
def get_filepath_or_buffer(filepath_or_buffer: FilePathOrBuffer[T], encoding: Optional[str]=..., compression: Optional[str]=..., mode: Optional[str]=...) -> Any: ...
def file_path_to_url(path: str) -> str: ...
def get_compression_method(compression: Optional[Union[str, Mapping[str, str]]]) -> Tuple[Optional[str], Dict[str, str]]: ...
def infer_compression(filepath_or_buffer: FilePathOrBuffer[T], compression: Optional[str]) -> Optional[str]: ...
def get_handle(path_or_buf: Any, mode: str, encoding: Any = ..., compression: Optional[Union[str, Mapping[str, Any]]]=..., memory_map: bool=..., is_text: bool=...) -> Any: ...

class _BytesZipFile(zipfile.ZipFile, BytesIO):  # type: ignore
    archive_name: Any = ...
    def __init__(self, file: FilePathOrBuffer[T], mode: str, archive_name: Optional[str]=..., **kwargs: Any) -> None: ...
    def write(self, data: Any) -> None: ...  # type: ignore
    @property
    def closed(self) -> Any: ...

class _MMapWrapper(abc.Iterator[T]):
    mmap: Any = ...
    def __init__(self, f: IO[AnyStr]) -> None: ...
    def __getattr__(self, name: str) -> Any: ...
    def __iter__(self) -> _MMapWrapper[T]: ...
    def __next__(self) -> str: ...  # type: ignore



from pandas.core.arrays import BooleanArray as BooleanArray, Categorical as Categorical, DatetimeArray as DatetimeArray, IntegerArray as IntegerArray, IntervalArray as IntervalArray, PandasArray as PandasArray, PeriodArray as PeriodArray, SparseArray as SparseArray, StringArray as StringArray, TimedeltaArray as TimedeltaArray



from pandas._config.config import options as options
from typing import Any, Optional

def set_locale(new_locale: Any, lc_var: Any = ...) -> None: ...
def can_set_locale(lc: Any, lc_var: Any = ...) -> Any: ...
def get_locales(prefix: Optional[Any] = ..., normalize: bool = ..., locale_getter: Any = ...) -> Any: ...



from pandas._config import get_option as get_option
from pandas.core.dtypes.common import ensure_object as ensure_object, is_bool_dtype as is_bool_dtype, is_complex_dtype as is_complex_dtype, is_datetime64_dtype as is_datetime64_dtype, is_datetime64tz_dtype as is_datetime64tz_dtype, is_datetimelike_v_numeric as is_datetimelike_v_numeric, is_dtype_equal as is_dtype_equal, is_extension_array_dtype as is_extension_array_dtype, is_float_dtype as is_float_dtype, is_integer_dtype as is_integer_dtype, is_object_dtype as is_object_dtype, is_period_dtype as is_period_dtype, is_scalar as is_scalar, is_string_dtype as is_string_dtype, is_string_like_dtype as is_string_like_dtype, is_timedelta64_dtype as is_timedelta64_dtype, needs_i8_conversion as needs_i8_conversion, pandas_dtype as pandas_dtype
from pandas.core.dtypes.generic import ABCDatetimeArray as ABCDatetimeArray, ABCExtensionArray as ABCExtensionArray, ABCGeneric as ABCGeneric, ABCIndexClass as ABCIndexClass, ABCMultiIndex as ABCMultiIndex, ABCSeries as ABCSeries, ABCTimedeltaArray as ABCTimedeltaArray
from pandas.core.dtypes.inference import is_list_like as is_list_like
from typing import Any

isposinf_scalar: Any
isneginf_scalar: Any

def isna(obj: Any) -> Any: ...
isnull = isna

def notna(obj: Any) -> Any: ...
notnull = notna

def array_equivalent(left: Any, right: Any, strict_nan: bool=...) -> bool: ...
def na_value_for_dtype(dtype: Any, compat: bool=...) -> Any: ...
def remove_na_arraylike(arr: Any) -> Any: ...
def is_valid_nat_for_dtype(obj: Any, dtype: Any) -> bool: ...



# Removed some imports that were necessary only for implementation
from pandas._libs.tslibs import  Timedelta as Timedelta, Timestamp as Timestamp
from pandas.core import nanops as nanops
from pandas.core.algorithms import checked_add_with_arr as checked_add_with_arr
from pandas.core.arrays import datetimelike as dtl

from pandas.core.dtypes.dtypes import DatetimeTZDtype as DatetimeTZDtype
from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame, ABCIndexClass as ABCIndexClass, ABCSeries as ABCSeries, ABCTimedeltaIndex as ABCTimedeltaIndex
from pandas.core.dtypes.missing import isna as isna
from pandas.tseries.frequencies import to_offset as to_offset
from pandas.tseries.offsets import Tick as Tick
from typing import Any, Optional

class TimedeltaArray(dtl.DatetimeLikeArrayMixin, dtl.TimelikeOps):
    __array_priority__: int = ...
    @property
    def dtype(self) -> Any: ...
    def __init__(self, values: Any, dtype: Any = ..., freq: Optional[Any] = ..., copy: bool = ...) -> None: ...
    def astype(self, dtype: Any, copy: bool = ...) -> Any: ...
    def sum(self, axis: Any = ..., dtype: Any = ..., out: Any = ..., keepdims: bool=..., initial: Any = ..., skipna: bool=..., min_count: int=...) -> Any: ...
    def std(self, axis: Any = ..., dtype: Any = ..., out: Any = ..., ddof: int=..., keepdims: bool=..., skipna: bool=...) -> Any: ...
    def median(self, axis: Any = ..., out: Any = ..., overwrite_input: bool=..., keepdims: bool=..., skipna: bool=...) -> Any: ...
    def __mul__(self, other: Any) -> Any: ...
    __rmul__: Any = ...
    def __truediv__(self, other: Any) -> Any: ...
    def __rtruediv__(self, other: Any) -> Any: ...
    def __floordiv__(self, other: Any) -> Any: ...
    def __rfloordiv__(self, other: Any) -> Any: ...
    def __mod__(self, other: Any) -> Any: ...
    def __rmod__(self, other: Any) -> Any: ...
    def __divmod__(self, other: Any) -> Any: ...
    def __rdivmod__(self, other: Any) -> Any: ...
    def __neg__(self) -> Any: ...
    def __pos__(self) -> Any: ...
    def __abs__(self) -> Any: ...
    def total_seconds(self) -> Any: ...
    def to_pytimedelta(self) -> Any: ...
    days: Any = ...
    seconds: Any = ...
    microseconds: Any = ...
    nanoseconds: Any = ...
    @property
    def components(self) -> Any: ...

def sequence_to_td64ns(data: Any, copy: bool = ..., unit: str = ..., errors: str = ...) -> Any: ...
def ints_to_td64ns(data: Any, unit: str = ...) -> Any: ...
def objects_to_td64ns(data: Any, unit: str = ..., errors: str = ...) -> Any: ...



from typing import Any, Optional

def convert_to_line_delimits(s: Any) -> Any: ...
def nested_to_record(ds: Any, prefix: str=..., sep: str=..., level: int=..., max_level: Optional[int]=...) -> Any: ...

json_normalize: Any
_json_normalize: Any



from pandas import Index as Index
from typing import Any, Callable, Dict, List, Optional, Sequence, Union

class ExcelCell:
    __fields__: Any = ...
    row: Any = ...
    col: Any = ...
    val: Any = ...
    style: Any = ...
    mergestart: Any = ...
    mergeend: Any = ...
    def __init__(self, row: int, col: int, val: Any, style: Any = ..., mergestart: Any = ..., mergeend: Any = ...) -> None: ...

class CSSToExcelConverter:
    inherited: Any = ...
    def __init__(self, inherited: Optional[str]=...) -> None: ...
    compute_css: Any = ...
    def __call__(self, declarations_str: str) -> Dict[str, Dict[str, str]]: ...
    def build_xlstyle(self, props: Dict[str, str]) -> Dict[str, Dict[str, str]]: ...
    VERTICAL_MAP: Any = ...
    def build_alignment(self, props: Any) -> Dict[str, Optional[Union[bool, str]]]: ...
    def build_border(self, props: Dict[str, str]) -> Dict[str, Dict[str, str]]: ...
    def build_fill(self, props: Dict[str, str]) -> Any: ...
    BOLD_MAP: Any = ...
    ITALIC_MAP: Any = ...
    def build_font(self, props: Any) -> Dict[str, Optional[Union[bool, int, str]]]: ...
    NAMED_COLORS: Any = ...
    def color_to_excel(self, val: Optional[str]) -> Any: ...
    def build_number_format(self, props: Dict[str, str]) -> Dict[str, Optional[str]]: ...

class ExcelFormatter:
    max_rows: Any = ...
    max_cols: Any = ...
    rowcounter: int = ...
    na_rep: Any = ...
    styler: Any = ...
    style_converter: Any = ...
    df: Any = ...
    columns: Any = ...
    float_format: Any = ...
    index: Any = ...
    index_label: Any = ...
    header: Any = ...
    merge_cells: Any = ...
    inf_rep: Any = ...
    def __init__(self, df: Any, na_rep: str=..., float_format: Optional[str]=..., cols: Optional[Sequence[Any]]=..., header: Union[bool, List[str]]=..., index: bool=..., index_label: Union[str, Sequence[Any], None]=..., merge_cells: bool=..., inf_rep: str=..., style_converter: Optional[Callable[..., Any]]=...) -> None: ...
    @property
    def header_style(self) -> Any: ...
    def get_formatted_cells(self) -> None: ...
    def write(self, writer: Any, sheet_name: str = ..., startrow: int = ..., startcol: int = ..., freeze_panes: Optional[Any] = ..., engine: Optional[Any] = ...) -> None: ...



from pandas import DataFrame as DataFrame
from pandas._typing import FilePathOrBuffer as FilePathOrBuffer
from pandas.io.common import get_filepath_or_buffer as get_filepath_or_buffer
from typing import Any, List, Optional, AnyStr


def read_orc(path: FilePathOrBuffer[AnyStr], columns: Optional[List[str]]=..., **kwargs: Any) -> DataFrame: ...



from pandas.io.excel._base import _BaseExcelReader
from typing import Any

class _XlrdReader(_BaseExcelReader):
    def __init__(self, filepath_or_buffer: Any) -> None: ...
    def load_workbook(self, filepath_or_buffer: Any) -> Any: ...
    @property
    def sheet_names(self) -> Any: ...
    def get_sheet_by_name(self, name: Any) -> Any: ...
    def get_sheet_by_index(self, index: Any) -> Any: ...
    def get_sheet_data(self, sheet: Any, convert_float: Any) -> Any: ...



import numpy as np
from .masked import BaseMaskedArray as BaseMaskedArray
from pandas._typing import Scalar as Scalar
from pandas.core import nanops as nanops, ops as ops
from pandas.core.dtypes.base import ExtensionDtype as ExtensionDtype
from pandas.core.dtypes.cast import astype_nansafe as astype_nansafe

from pandas.core.dtypes.dtypes import register_extension_dtype as register_extension_dtype
from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame, ABCIndexClass as ABCIndexClass, ABCSeries as ABCSeries
from pandas.core.dtypes.missing import isna as isna, notna as notna

from typing import Any, Type

class BooleanDtype(ExtensionDtype):
    name: str = ...
    @property
    def na_value(self) -> Scalar: ...
    @property
    def type(self) -> Type[Any]: ...
    @property
    def kind(self) -> str: ...
    @classmethod
    def construct_array_type(cls: Any) -> Type[BooleanArray]: ...
    def __from_arrow__(self, array: Any) -> Any: ...

def coerce_to_array(values: Any, mask: Any = ..., copy: bool=...) -> Any: ...

class BooleanArray(BaseMaskedArray):
    def __init__(self, values: np.ndarray, mask: np.ndarray, copy: bool=...) -> None: ...
    @property
    def dtype(self) -> Any: ...
    def __array_ufunc__(self, ufunc: Any, method: Any, *inputs: Any, **kwargs: Any) -> Any: ...
    def __setitem__(self, key: Any, value: Any) -> None: ...
    def astype(self, dtype: Any, copy: bool = ...) -> Any: ...
    def any(self, skipna: bool=..., **kwargs: Any) -> Any: ...
    def all(self, skipna: bool=..., **kwargs: Any) -> Any: ...



from pandas import DataFrame as DataFrame
from typing import Any, Dict, List, Optional

def read_gbq(query: str, project_id: Optional[str]=..., index_col: Optional[str]=..., col_order: Optional[List[str]]=..., reauth: bool=..., auth_local_webserver: bool=..., dialect: Optional[str]=..., location: Optional[str]=..., configuration: Optional[Dict[str, Any]]=..., credentials: Any = ..., use_bqstorage_api: Optional[bool]=..., private_key: Any = ..., verbose: Any = ..., progress_bar_type: Optional[str]=...) -> DataFrame: ...
def to_gbq(dataframe: DataFrame, destination_table: str, project_id: Optional[str]=..., chunksize: Optional[int]=..., reauth: bool=..., if_exists: str=..., auth_local_webserver: bool=..., table_schema: Optional[List[Dict[str, str]]]=..., location: Optional[str]=..., progress_bar: bool=..., credentials: Any = ..., verbose: Any = ..., private_key: Any = ...) -> None: ...



from .masked import BaseMaskedArray as BaseMaskedArray
from pandas.core import nanops as nanops, ops as ops
from pandas.core.dtypes.base import ExtensionDtype as ExtensionDtype
from pandas.core.dtypes.cast import astype_nansafe as astype_nansafe

from pandas.core.dtypes.dtypes import register_extension_dtype as register_extension_dtype
from pandas.core.dtypes.missing import isna as isna

from pandas.core.ops import invalid_comparison as invalid_comparison
from pandas.core.ops.common import unpack_zerodim_and_defer as unpack_zerodim_and_defer
from pandas.core.tools.numeric import to_numeric as to_numeric
from pandas.util._decorators import cache_readonly as cache_readonly
from typing import Any, Optional, Type

class _IntegerDtype(ExtensionDtype):
    name: str
    base: Any = ...
    type: Type[Any]
    na_value: Any = ...
    def is_signed_integer(self) -> Any: ...
    def is_unsigned_integer(self) -> Any: ...
    def numpy_dtype(self) -> Any: ...
    def kind(self) -> Any: ...
    def itemsize(self) -> Any: ...
    @classmethod
    def construct_array_type(cls) -> Any: ...
    def __from_arrow__(self, array: Any) -> Any: ...

def integer_array(values: Any, dtype: Optional[Any] = ..., copy: bool = ...) -> Any: ...
def safe_cast(values: Any, dtype: Any, copy: Any) -> Any: ...
def coerce_to_array(values: Any, dtype: Any, mask: Optional[Any] = ..., copy: bool = ...) -> Any: ...

class IntegerArray(BaseMaskedArray):
    def dtype(self) -> Any: ...
    def __init__(self, values: Any, mask: Any, copy: bool = ...) -> None: ...
    def __array_ufunc__(self, ufunc: Any, method: Any, *inputs: Any, **kwargs: Any) -> Any: ...
    def __setitem__(self, key: Any, value: Any) -> None: ...
    def astype(self, dtype: Any, copy: bool = ...) -> Any: ...

Int8Dtype: Any
Int16Dtype: Any
Int32Dtype: Any
Int64Dtype: Any
UInt8Dtype: Any
UInt16Dtype: Any
UInt32Dtype: Any
UInt64Dtype: Any



import numpy as np
from pandas.core.base import NoNewAttributesMixin as NoNewAttributesMixin
from pandas.core.indexes.base import Index
from pandas.core.series import Series
from typing import Any, List, Optional, Union


def cat_core(list_of_columns: List[np.ndarray], sep: str) -> Any: ...
def cat_safe(list_of_columns: List[np.ndarray], sep: str) -> Any: ...
def str_count(arr: Any, pat: Any, flags: int = ...) -> Any: ...
def str_contains(arr: Any, pat: Any, case: bool = ..., flags: int = ..., na: Any = ..., regex: bool = ...) -> Any: ...
def str_startswith(arr: Any, pat: Any, na: Any = ...) -> Any: ...
def str_endswith(arr: Any, pat: Any, na: Any = ...) -> Any: ...
def str_replace(arr: Any, pat: Any, repl: Any, n: int = ..., case: Optional[Any] = ..., flags: int = ..., regex: bool = ...) -> Any: ...
def str_repeat(arr: Any, repeats: Any) -> Any: ...
def str_match(arr: Any, pat: Any, case: bool = ..., flags: int = ..., na: Any = ...) -> Any: ...
def str_extract(arr: Any, pat: Any, flags: int = ..., expand: bool = ...) -> Any: ...
def str_extractall(arr: Any, pat: Any, flags: int = ...) -> Any: ...
def str_get_dummies(arr: Any, sep: str = ...) -> Any: ...
def str_join(arr: Any, sep: Any) -> Any: ...
def str_findall(arr: Any, pat: Any, flags: int = ...) -> Any: ...
def str_find(arr: Any, sub: Any, start: int = ..., end: Optional[Any] = ..., side: str = ...) -> Any: ...
def str_index(arr: Any, sub: Any, start: int = ..., end: Optional[Any] = ..., side: str = ...) -> Any: ...
def str_pad(arr: Any, width: Any, side: str = ..., fillchar: str = ...) -> Any: ...
def str_split(arr: Any, pat: Optional[Any] = ..., n: Optional[Any] = ...) -> Any: ...
def str_rsplit(arr: Any, pat: Optional[Any] = ..., n: Optional[Any] = ...) -> Any: ...
def str_slice(arr: Any, start: Optional[Any] = ..., stop: Optional[Any] = ..., step: Optional[Any] = ...) -> Any: ...
def str_slice_replace(arr: Any, start: Optional[Any] = ..., stop: Optional[Any] = ..., repl: Optional[Any] = ...) -> Any: ...
def str_strip(arr: Any, to_strip: Optional[Any] = ..., side: str = ...) -> Any: ...
def str_wrap(arr: Any, width: Any, **kwargs: Any) -> Any: ...
def str_translate(arr: Any, table: Any) -> Any: ...
def str_get(arr: Any, i: Any) -> Any: ...
def str_decode(arr: Any, encoding: Any, errors: str = ...) -> Any: ...
def str_encode(arr: Any, encoding: Any, errors: str = ...) -> Any: ...
def forbid_nonstring_types(forbidden: Any, name: Optional[Any] = ...) -> Any: ...
def copy(source: Any) -> Any: ...

class StringMethods(NoNewAttributesMixin):
    def __init__(self, data: Any) -> None: ...
    def __getitem__(self, key: Any) -> Any: ...
    def __iter__(self) -> Any: ...
    def cat(self, others: Optional[Any] = ..., sep: Optional[Any] = ..., na_rep: Optional[Any] = ..., join: str = ...) -> Any: ...
    def split(self, pat: Optional[Any] = ..., n: int = ..., expand: bool = ...) -> Any: ...
    def rsplit(self, pat: Optional[Any] = ..., n: int = ..., expand: bool = ...) -> Any: ...
    def partition(self, sep: str = ..., expand: bool = ...) -> Any: ...
    def rpartition(self, sep: str = ..., expand: bool = ...) -> Any: ...
    def get(self, i: Any) -> Any: ...
    def join(self, sep: Any) -> Any: ...
    def contains(self, pat: Any, case: bool = ..., flags: int = ..., na: Any = ..., regex: bool = ...) -> Any: ...
    def match(self, pat: Any, case: bool = ..., flags: int = ..., na: Any = ...) -> Any: ...
    def replace(self, pat: Any, repl: Any, n: int = ..., case: Optional[Any] = ..., flags: int = ..., regex: bool = ...) -> Any: ...
    def repeat(self, repeats: Any) -> Any: ...
    def pad(self, width: Any, side: str = ..., fillchar: str = ...) -> Any: ...
    def center(self, width: Any, fillchar: str = ...) -> Any: ...
    def ljust(self, width: Any, fillchar: str = ...) -> Any: ...
    def rjust(self, width: Any, fillchar: str = ...) -> Any: ...
    def zfill(self, width: Any) -> Any: ...
    def slice(self, start: Optional[Any] = ..., stop: Optional[Any] = ..., step: Optional[Any] = ...) -> Any: ...
    def slice_replace(self, start: Optional[Any] = ..., stop: Optional[Any] = ..., repl: Optional[Any] = ...) -> Any: ...
    def decode(self, encoding: Any, errors: str = ...) -> Any: ...
    def encode(self, encoding: Any, errors: str = ...) -> Any: ...
    def strip(self, to_strip: Optional[str] = ...) -> Union[Series, Index]: ...
    def lstrip(self, to_strip: Optional[str] = ...) -> Union[Series, Index]: ...
    def rstrip(self, to_strip: Optional[str] = ...) -> Union[Series, Index]: ...
    def wrap(self, width: Any, **kwargs: Any) -> Any: ...
    def get_dummies(self, sep: str = ...) -> Any: ...
    def translate(self, table: Any) -> Any: ...
    count: Any = ...
    startswith: Any = ...
    endswith: Any = ...
    findall: Any = ...
    def extract(self, pat: Any, flags: int = ..., expand: bool = ...) -> Any: ...
    def extractall(self, pat: Any, flags: int = ...) -> Any: ...
    def find(self, sub: Any, start: int = ..., end: Optional[Any] = ...) -> Any: ...
    def rfind(self, sub: Any, start: int = ..., end: Optional[Any] = ...) -> Any: ...
    def normalize(self, form: Any) -> Any: ...
    def index(self, sub: Any, start: int = ..., end: Optional[Any] = ...) -> Any: ...
    def rindex(self, sub: Any, start: int = ..., end: Optional[Any] = ...) -> Any: ...
    len: Any = ...
    lower: Any = ...
    upper: Any = ...
    title: Any = ...
    capitalize: Any = ...
    swapcase: Any = ...
    casefold: Any = ...
    isalnum: Any = ...
    isalpha: Any = ...
    isdigit: Any = ...
    isspace: Any = ...
    islower: Any = ...
    isupper: Any = ...
    istitle: Any = ...
    isnumeric: Any = ...
    isdecimal: Any = ...



# Removed some imports that were necessary only for implementation
from pandas.core import algorithms as algorithms
from pandas.core.accessor import PandasDelegate as PandasDelegate
from pandas.core.arrays import DatetimeArray as DatetimeArray, ExtensionArray as ExtensionArray, TimedeltaArray as TimedeltaArray
from pandas.core.arrays.datetimelike import DatetimeLikeArrayMixin as DatetimeLikeArrayMixin
from pandas.core.dtypes.generic import ABCIndex as ABCIndex, ABCIndexClass as ABCIndexClass, ABCSeries as ABCSeries
from pandas.core.dtypes.missing import isna as isna
from pandas.core.indexes.base import Index as Index
from pandas.core.indexes.extension import ExtensionIndex as ExtensionIndex
from pandas.core.indexes.numeric import Int64Index as Int64Index
from pandas.core.ops import get_op_result_name as get_op_result_name
from pandas.core.tools.timedeltas import to_timedelta as to_timedelta
from pandas.errors import AbstractMethodError as AbstractMethodError
from pandas.tseries.frequencies import DateOffset as DateOffset, to_offset as to_offset
from pandas.util._decorators import Appender as Appender, cache_readonly as cache_readonly
from typing import Any, List, Optional

class DatetimeIndexOpsMixin(ExtensionIndex):
    freq: Optional[DateOffset]
    freqstr: Optional[str]
    hasnans: Any = ...
    @property
    def is_all_dates(self) -> bool: ...
    @property
    def values(self) -> Any: ...
    def __array_wrap__(self, result: Any, context: Optional[Any] = ...) -> Any: ...
    def equals(self, other: Any) -> bool: ...
    def __contains__(self, key: Any) -> Any: ...
    def sort_values(self, return_indexer: bool = ..., ascending: bool = ...) -> Any: ...
    def take(self, indices: Any, axis: int = ..., allow_fill: bool = ..., fill_value: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def tolist(self) -> List[Any]: ...
    def min(self, axis: Optional[Any] = ..., skipna: bool = ..., *args: Any, **kwargs: Any) -> Any: ...
    def argmin(self, axis: Optional[Any] = ..., skipna: bool = ..., *args: Any, **kwargs: Any) -> Any: ...
    def max(self, axis: Optional[Any] = ..., skipna: bool = ..., *args: Any, **kwargs: Any) -> Any: ...
    def argmax(self, axis: Optional[Any] = ..., skipna: bool = ..., *args: Any, **kwargs: Any) -> Any: ...
    __add__: Any = ...
    __radd__: Any = ...
    __sub__: Any = ...
    __rsub__: Any = ...
    __pow__: Any = ...
    __rpow__: Any = ...
    __mul__: Any = ...
    __rmul__: Any = ...
    __floordiv__: Any = ...
    __rfloordiv__: Any = ...
    __mod__: Any = ...
    __rmod__: Any = ...
    __divmod__: Any = ...
    __rdivmod__: Any = ...
    __truediv__: Any = ...
    __rtruediv__: Any = ...
    def isin(self, values: Any, level: Optional[Any] = ...) -> Any: ...
    def where(self, cond: Any, other: Optional[Any] = ...) -> Any: ...
    def shift(self, periods: int = ..., freq: Optional[Any] = ...) -> Any: ...
    def delete(self, loc: Any) -> Any: ...

class DatetimeTimedeltaMixin(DatetimeIndexOpsMixin, Int64Index):
    def difference(self, other: Any, sort: Optional[Any] = ...) -> Any: ...
    def intersection(self, other: Any, sort: bool = ...) -> Any: ...
    def join(self, other: Any, how: str=..., level: Any = ..., return_indexers: Any = ..., sort: Any = ...) -> Any: ...

class DatetimelikeDelegateMixin(PandasDelegate): ...



from typing import Any

def cut(x: Any, bins: Any, right: bool=..., labels: Any = ..., retbins: bool=..., precision: int=..., include_lowest: bool=..., duplicates: str=...) -> Any: ...
def qcut(x: Any, q: Any, labels: Any = ..., retbins: bool=..., precision: int=..., duplicates: str=...) -> Any: ...



from pandas._config.config import option_context as option_context
from pandas._libs import Timestamp as Timestamp
from pandas._typing import FrameOrSeries as FrameOrSeries, Scalar as Scalar
from pandas.core import nanops as nanops
from pandas.core.arrays import Categorical as Categorical, DatetimeArray as DatetimeArray, try_cast_to_ea as try_cast_to_ea
from pandas.core.base import DataError as DataError, PandasObject as PandasObject, SelectionMixin as SelectionMixin
from pandas.core.dtypes.cast import maybe_downcast_to_dtype as maybe_downcast_to_dtype
from pandas.core.dtypes.common import ensure_float as ensure_float, is_datetime64_dtype as is_datetime64_dtype, is_extension_array_dtype as is_extension_array_dtype, is_integer_dtype as is_integer_dtype, is_numeric_dtype as is_numeric_dtype, is_object_dtype as is_object_dtype, is_scalar as is_scalar
from pandas.core.dtypes.missing import isna as isna, notna as notna
from pandas.core.frame import DataFrame as DataFrame
from pandas.core.generic import NDFrame as NDFrame
from pandas.core.groupby import base as base, ops as ops
from pandas.core.indexes.api import CategoricalIndex as CategoricalIndex, Index as Index, MultiIndex as MultiIndex
from pandas.core.series import Series as Series
from pandas.core.sorting import get_group_index_sorter as get_group_index_sorter
from pandas.errors import AbstractMethodError as AbstractMethodError
from pandas.util._decorators import Appender as Appender, Substitution as Substitution, cache_readonly as cache_readonly
from typing import Any, List, Optional, Union, Hashable, Callable, Mapping, TypeVar, Tuple, overload

PipeReturn = TypeVar('PipeReturn')


class GroupByPlot(PandasObject):
    def __init__(self, groupby: Any) -> None: ...
    def __call__(self, *args: Any, **kwargs: Any) -> Any: ...
    def __getattr__(self, name: str) -> Any: ...

_KeysArgType = Union[
    Hashable,
    List[Hashable],
    Callable[[Hashable], Hashable],
    List[Callable[[Hashable], Hashable]],
    Mapping[Hashable, Hashable],
]

class _GroupBy(PandasObject, SelectionMixin):
    level: Any = ...
    as_index: Any = ...
    keys: Any = ...
    sort: Any = ...
    group_keys: Any = ...
    squeeze: Any = ...
    observed: Any = ...
    mutated: Any = ...
    obj: Any = ...
    axis: Any = ...
    grouper: Any = ...
    exclusions: Any = ...
    def __init__(self, obj: NDFrame, keys: Optional[_KeysArgType]=..., axis: int=..., level: Any = ..., grouper: Optional[ops.BaseGrouper]=..., exclusions: Any = ..., selection: Any = ..., as_index: bool=..., sort: bool=..., group_keys: bool=..., squeeze: bool=..., observed: bool=..., mutated: bool=...) -> None: ...
    def __len__(self) -> int: ...
    @property
    def groups(self) -> Any: ...
    @property
    def ngroups(self) -> Any: ...
    @property
    def indices(self) -> Any: ...
    def __getattr__(self, attr: str) -> Any: ...
    @overload
    def pipe(self: Any, func: Union[Callable[..., PipeReturn], Tuple[Callable[..., PipeReturn], str]], *args: Any, **kwargs: Any) -> PipeReturn: ...
    @overload
    def pipe(self: Any, func: PipeReturn, *args: Any, **kwargs: Any) -> PipeReturn: ...
    plot: Any = ...
    def get_group(self, name: Any, obj: Optional[Any] = ...) -> Any: ...
    def __iter__(self) -> Any: ...
    def apply(self, func: Any, *args: Any, **kwargs: Any) -> Any: ...
    def transform(self, func: Any, *args: Any, **kwargs: Any) -> None: ...

class GroupBy(_GroupBy):
    def any(self, skipna: bool=...) -> Any: ...
    def all(self, skipna: bool=...) -> Any: ...
    def count(self) -> None: ...
    def mean(self, *args: Any, **kwargs: Any) -> Any: ...
    def median(self, **kwargs: Any) -> Any: ...
    def std(self, ddof: int=..., *args: Any, **kwargs: Any) -> Any: ...
    def var(self, ddof: int=..., *args: Any, **kwargs: Any) -> Any: ...
    def sem(self, ddof: int=...) -> Any: ...
    def size(self) -> Any: ...
    def ohlc(self) -> DataFrame: ...
    def describe(self, **kwargs: Any) -> Any: ...
    def resample(self, rule: Any, *args: Any, **kwargs: Any) -> Any: ...
    def rolling(self, *args: Any, **kwargs: Any) -> Any: ...
    def expanding(self, *args: Any, **kwargs: Any) -> Any: ...
    def pad(self, limit: Optional[Any] = ...) -> Any: ...
    ffill: Any = ...
    def backfill(self, limit: Optional[Any] = ...) -> Any: ...
    bfill: Any = ...
    def nth(self, n: Union[int, List[int]], dropna: Optional[str]=...) -> DataFrame: ...
    def quantile(self, q: Any = ..., interpolation: str=...) -> Any: ...
    def ngroup(self, ascending: bool=...) -> Any: ...
    def cumcount(self, ascending: bool=...) -> Any: ...
    def rank(self, method: str=..., ascending: bool=..., na_option: str=..., pct: bool=..., axis: int=...) -> Any: ...
    def cumprod(self, axis: int = ..., *args: Any, **kwargs: Any) -> Any: ...
    def cumsum(self, axis: int = ..., *args: Any, **kwargs: Any) -> Any: ...
    def cummin(self, axis: int = ..., **kwargs: Any) -> Any: ...
    def cummax(self, axis: int = ..., **kwargs: Any) -> Any: ...
    def shift(self, periods: int = ..., freq: Optional[Any] = ..., axis: int = ..., fill_value: Optional[Any] = ...) -> Any: ...
    def pct_change(self, periods: int = ..., fill_method: str = ..., limit: Optional[Any] = ..., freq: Optional[Any] = ..., axis: int = ...) -> Any: ...
    def head(self, n: int = ...) -> Any: ...
    def tail(self, n: int = ...) -> Any: ...

def get_groupby(obj: NDFrame, by: Optional[_KeysArgType]=..., axis: int=..., level: Any = ..., grouper: Optional[ops.BaseGrouper]=..., exclusions: Any = ..., selection: Any = ..., as_index: bool=..., sort: bool=..., group_keys: bool=..., squeeze: bool=..., observed: bool=..., mutated: bool=...) -> GroupBy: ...



from pandas._typing import FilePathOrBuffer as FilePathOrBuffer
from pandas.core.dtypes.generic import ABCDatetimeIndex as ABCDatetimeIndex, ABCIndexClass as ABCIndexClass, ABCMultiIndex as ABCMultiIndex, ABCPeriodIndex as ABCPeriodIndex
from pandas.core.dtypes.missing import notna as notna
from pandas.io.common import get_compression_method as get_compression_method, get_filepath_or_buffer as get_filepath_or_buffer, get_handle as get_handle, infer_compression as infer_compression
from typing import Any, Hashable, Mapping, Optional, Sequence, Union

class CSVFormatter:
    obj: Any = ...
    sep: Any = ...
    na_rep: Any = ...
    float_format: Any = ...
    decimal: Any = ...
    header: Any = ...
    index: Any = ...
    index_label: Any = ...
    mode: Any = ...
    encoding: Any = ...
    compression: Any = ...
    quoting: Any = ...
    quotechar: Any = ...
    doublequote: Any = ...
    escapechar: Any = ...
    line_terminator: Any = ...
    date_format: Any = ...
    has_mi_columns: Any = ...
    cols: Any = ...
    blocks: Any = ...
    data: Any = ...
    chunksize: Any = ...
    data_index: Any = ...
    nlevels: Any = ...
    def __init__(self, obj: Any, path_or_buf: Optional[FilePathOrBuffer[str]]=..., sep: str=..., na_rep: str=..., float_format: Optional[str]=..., cols: Any = ..., header: Union[bool, Sequence[Hashable]]=..., index: bool=..., index_label: Optional[Union[bool, Hashable, Sequence[Hashable]]]=..., mode: str=..., encoding: Optional[str]=..., compression: Union[str, Mapping[str, str], None]=..., quoting: Optional[int]=..., line_terminator: Any = ..., chunksize: Optional[int]=..., quotechar: Any = ..., date_format: Optional[str]=..., doublequote: bool=..., escapechar: Optional[str]=..., decimal: Any = ...) -> None: ...
    writer: Any = ...
    def save(self) -> None: ...
